# (C) Copyright 2015-2016 Hewlett Packard Enterprise Development LP
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import itertools
import logging
import os

from monasca_setup import agent_config
from monasca_setup.detection import Plugin
from monasca_setup.detection.utils import find_process_cmdline
from monasca_setup.detection.utils import watch_process

log = logging.getLogger(__name__)


class Ceph(Plugin):

    """Detect Ceph daemons and setup configuration to monitor them.

    The Ceph services comprise of three main daemons:
      - Object Storage Daemon (OSD)
          There is a OSD service process per disk added in the cluster
          and has a corresponding daemon identifier at /var/lib/ceph/osd/
      - Monitor Servers (MON)
          There is a Monitor service process per monitoring host in the
          cluster and has a corresponding daemon identifier at
          /var/lib/ceph/mon/
      - Metadata Servers (MDS)
          There is a MDS service process per monitoring host in the
          cluster and has a corresponding daemon identifier at
          /var/lib/ceph/mds/

    The ceph object store is exposed via RADOS Gateway
      - RADOS Gateway (radosgw)
          'radosgw' is an HTTP REST gateway for the RADOS object store,
          a part of the Ceph distributed storage system.

    Unlike OpenStack services monitoring a single instance of a service will
    be of no use for Ceph. Every daemon needs to be monitored. This can be
    achieved by scanning through the daemon identifiers.

    """

    def __init__(self, template_dir, overwrite=True, args=None):
        self.service_name = 'ceph-storage'
        self.process_names = ['ceph-osd', 'ceph-mon', 'ceph-mds', 'radosgw']
        self.ceph_config_dir = '/etc/ceph/'
        self.service_constants = dict()
        for process in self.process_names:
            process_type = process.replace('ceph-', '')
            display_name = 'ceph-%s' % process \
                if not process.startswith('ceph-') else process
            self.service_constants[process_type] = {
                'service_dir': '/var/lib/ceph/%s/' % process_type,
                'executable': '/usr/bin/%s' % process,
                'display_name': display_name
            }

        super(Ceph, self).__init__(template_dir, overwrite, args)

    def _detect(self):
        """Run detection.

        """
        self.found_processes = list()

        for process in self.process_names:
            if find_process_cmdline(process) is not None:
                self.found_processes.append(process)
        if len(self.found_processes) > 0:
            self.available = True

    @staticmethod
    def _build_search_string(executable, options=None):
        search_strings = list()
        command = [executable]
        if options:
            command.extend(options)

        for permutation in itertools.permutations(command):
            if permutation[0] == executable:
                search_strings.append(" ".join(permutation))
        return search_strings

    def _service_config(self, clusters, service_type):
        display_name = self.service_constants[service_type]['display_name']
        service_dir = self.service_constants[service_type]['service_dir']
        executable = self.service_constants[service_type]['executable']
        expected_processes = list()

        for cluster in clusters:
            cluster_name = cluster['cluster_name']
            instance_list = list()

            # Get the list of daemon identifiers for given cluster
            if os.path.exists(service_dir):
                instance_list = [entry for entry in os.listdir(service_dir)
                                 if entry.startswith(cluster_name)]

            for instance in instance_list:
                # Daemon identifier is of format <cluster_name>-<id>
                # 'id' for ceph-mon is alphanumeric and is usually the hostname
                # where the service is running for ceph-mon
                # E.g., ceph-monitor1.dom, ceph-monitor2.dom etc.
                #
                # 'id' for ceph-osd is a unique numeric index for
                # that OSD in the cluster
                # E.g., ceph-1, ceph-2 etc.
                #
                # 'id' for ceph-mds is alphanumeric and is usually the hostname
                # where the service is running.
                # E.g., ceph-mds1.dom, ceph-mds2.dom etc.
                daemon_id = instance.split(cluster_name + '-', 1)[1]
                process = dict()
                process_args = ['--cluster %s' % cluster_name,
                                '--id %s' % daemon_id, '-f']
                process['search_string'] = self._build_search_string(
                    executable, process_args)
                process['name'] = '%s-%s.%s' \
                                  % (cluster_name, service_type, daemon_id)
                process['type'] = display_name
                expected_processes.append(process)

        return expected_processes

    def _radosgw_config(self, clusters):
        service_dir = self.service_constants['radosgw']['service_dir']
        expected_processes = list()

        for cluster in clusters:
            cluster_name = cluster['cluster_name']
            config_file = cluster['config_file']
            instance_list = list()

            # Get the list of daemon identifiers for given cluster
            if os.path.exists(service_dir):
                instance_list = [entry for entry in os.listdir(service_dir)
                                 if entry.startswith(cluster_name)]

            for instance in instance_list:
                # RADOS Gateway processes is of the format:
                # /usr/bin/radosgw -c <config_file> -n <rados_username>
                # E.g.,
                # /usr/bin/radosgw -c ceph.conf -n client.radosgw.gateway
                process = dict()

                # The rados user will have a designated data directory, of the
                # format ceph-radosw.<rados_username> in the service dir.
                # E.g., /var/lib/ceph/radosgw/ceph-radosgw.gateway
                rados_username = instance.replace('ceph-radosgw.', '')
                process['search_string'] = list()
                process['name'] = '%s-radosgw.%s' \
                                  % (cluster_name, rados_username)
                process['type'] = \
                    self.service_constants['radosgw']['display_name']
                executable = self.service_constants['radosgw']['executable']

                process_options = ['-n client.radosgw.%s' % rados_username,
                                   '--name=client.radosgw.%s' % rados_username]
                for opt in process_options:
                    # Adding multiple combinations for all possible use cases,
                    # since any of the following combination can be used to
                    # start the process

                    # Trivial case (This will be the most used scenario)
                    # E.g.,
                    # /usr/bin/radosgw -n client.radosgw.gateway
                    process['search_string'].append(
                        '%s %s' % (executable, opt))

                    # Service started with specific conf file (For rare cases)
                    # E.g.,
                    # /usr/bin/radosgw -c custom.conf -n client.radosgw.gateway
                    process['search_string'].append(
                        '%s -c %s %s' % (executable, config_file, opt))
                    process['search_string'].append(
                        '%s --conf=%s %s' % (executable, config_file, opt))
                expected_processes.append(process)

        return expected_processes

    def build_config(self):
        """Build the config as a Plugins object and return.

        """
        config = agent_config.Plugins()

        # There may be multiple clusters, and we construct a list of dicts
        # containing cluster_name and config_file for each cluster
        clusters = list()

        # Get the cluster_name from <cluster_name>.conf in /etc/ceph/ directory
        if os.path.exists(self.ceph_config_dir):
            config_files = [f for f in os.listdir(self.ceph_config_dir)
                            if f.endswith('.conf')]
            if not config_files:
                return config
            for config_file in config_files:
                cluster_dict = dict()
                cluster_dict['cluster_name'] = config_file[:-5]
                cluster_dict['config_file'] = \
                    os.path.join(self.ceph_config_dir, config_file)
                clusters.append(cluster_dict)

        expected_processes = list()

        expected_processes.extend(self._service_config(clusters, 'mon'))
        expected_processes.extend(self._service_config(clusters, 'osd'))
        expected_processes.extend(self._service_config(clusters, 'mds'))
        # RADOS Gateway is little different from other ceph-daemons hence
        # the process definition is handled differently
        expected_processes.extend(self._radosgw_config(clusters))

        for process in expected_processes:
            # Watch the service processes
            log.info("\tMonitoring the {0} {1} process.".format(
                process['name'], self.service_name))
            config.merge(watch_process(search_strings=process['search_string'],
                                       service=self.service_name,
                                       component=process['type'],
                                       process_name=process['name'],
                                       exact_match=False))

        return config



# (C) Copyright 2015 Hewlett Packard Enterprise Development Company LP

import logging
import re
import subprocess
import sys

import monasca_agent.collector.checks as checks
from monasca_agent.common.psutil_wrapper import psutil
import monasca_agent.common.util as util


log = logging.getLogger(__name__)


class Load(checks.AgentCheck):

    def __init__(self, name, init_config, agent_config):
        super(Load, self).__init__(name, init_config, agent_config)

    def check(self, instance):
        """Capture load stats

        """

        dimensions = self._set_dimensions(None, instance)

        if util.Platform.is_linux():
            try:
                loadAvrgProc = open('/proc/loadavg', 'r')
                uptime = loadAvrgProc.readlines()
                loadAvrgProc.close()
            except Exception:
                log.exception('Cannot extract load using /proc/loadavg')
                return

            uptime = uptime[0]  # readlines() provides a list but we want a string

        elif sys.platform in ('darwin', 'sunos5') or sys.platform.startswith("freebsd"):
            # Get output from uptime
            try:
                uptime = subprocess.Popen(['uptime'],
                                          stdout=subprocess.PIPE,
                                          close_fds=True).communicate()[0]
            except Exception:
                log.exception('Cannot extract load using uptime')
                return

        # Split out the 3 load average values
        load = [res.replace(',', '.') for res in re.findall(r'([0-9]+[\.,]\d+)', uptime)]

        dimensions = self._set_dimensions(None)

        #
        # Normalize the load averages by number of cores
        # so the metric is useful for alarming across
        # hosts with varying core numbers
        #
        num_cores = psutil.cpu_count(logical=True)

        self.gauge('load.avg_1_min',
                   round((float(load[0]) / num_cores), 3),
                   dimensions=dimensions)
        self.gauge('load.avg_5_min',
                   round((float(load[1]) / num_cores), 3),
                   dimensions=dimensions)
        self.gauge('load.avg_15_min',
                   round((float(load[2]) / num_cores), 3),
                   dimensions=dimensions)

        log.debug("Collected 3 load metrics (normalized by {} cores)".format(num_cores))



import nose.tools as nt

from monasca_agent.collector.checks import AgentCheck
from monasca_agent.common.check_status import STATUS_OK, STATUS_ERROR, InstanceStatus, CheckStatus, CollectorStatus


class DummyAgentCheck(AgentCheck):

    @staticmethod
    def check(instance):
        if not instance['pass']:
            raise Exception("failure")


def test_check_status_fail():
    instances = [
        {'pass': True},
        {'pass': False},
        {'pass': True}
    ]

    check = DummyAgentCheck('dummy_agent_check', {}, {}, instances)
    instance_statuses = check.run()
    assert len(instance_statuses) == 3
    assert instance_statuses[0].status == STATUS_OK
    assert instance_statuses[1].status == STATUS_ERROR
    assert instance_statuses[2].status == STATUS_OK


def test_check_status_pass():
    instances = [
        {'pass': True},
        {'pass': True},
    ]

    check = DummyAgentCheck('dummy_agent_check', {}, {}, instances)
    instances_status = check.run()
    assert len(instances_status) == 2
    for i in instances_status:
        assert i.status == STATUS_OK


def test_persistence():
    i1 = InstanceStatus(1, STATUS_OK)
    chk1 = CheckStatus("dummy", [i1], 1, 2)
    c1 = CollectorStatus([chk1])
    c1.persist()

    c2 = CollectorStatus.load_latest_status()
    nt.assert_equal(1, len(c2.check_statuses))
    chk2 = c2.check_statuses[0]
    assert chk2.name == chk1.name
    assert chk2.status == chk2.status
    assert chk2.metric_count == 1
    assert chk2.event_count == 2


def test_persistence_fail():

    # Assert remove doesn't crap out if a file doesn't exist.
    CollectorStatus.remove_latest_status()
    CollectorStatus.remove_latest_status()

    status = CollectorStatus.load_latest_status()
    assert not status



# (C) Copyright 2016 Hewlett Packard Enterprise Development LP

import json
import logging
import yaml

import monasca_setup.agent_config
import monasca_setup.detection
from monasca_setup.detection.utils import find_process_cmdline

log = logging.getLogger(__name__)


class ProcessCheck(monasca_setup.detection.Plugin):
    """Setup a process check according to the passed in JSON string or YAML config file path.

       A process can be monitored by process_names or by process_username, or by both if
       the process_config list contains both dictionary entries. Pass in the dictionary containing process_names
       when watching process by name.  Pass in the dictionary containing process_username and dimensions with
       component when watching process by username. Watching by process_username is useful for groups of processes
       that are owned by a specific user. For process monitoring by process_username the component dimension
       is required since it is used to initialize the instance name in process.yaml.

       service and component dimensions are recommended to distinguish multiple components per service.  The component
       dimensions will be defaulted to the process name when it is not input when monitoring by process_names.
       exact_match is optional and defaults to false, meaning the process name search string can be found within the process name.
       exact_match can be set to true if the process_names search string should match the process name.

       Pass in a YAML config file path:
       monasca-setup -d ProcessCheck -a "conf_file_path=/home/stack/myprocess.yaml"

       or

       Pass in a JSON string command line argument:
       Using monasca-setup, you can pass in a json string with arguments --detection_args_json, or the shortcut -json.

       Monitor by process_names:
       monasca-setup -d ProcessCheck -json \
         '{"process_config":[{"process_names":["monasca-notification","monasca-api"],"dimensions":{"service":"monitoring"}}]}'

       Specifiy one or more dictionary entries to the process_config list:
       monasca-setup -d ProcessCheck -json \
         '{"process_config":[
            {"process_names":["monasca-notification","monasca-api"],"dimensions":{"service":"monitoring"}},
            {"process_names":["elasticsearch"],"dimensions":{"service":"logging"},"exact_match":"true"},
            {"process_names":["monasca-thresh"],"dimensions":{"service":"monitoring","component":"thresh"}}]}'

       Monitor by process_username:
       monasca-setup -d ProcessCheck -json \
         '{"process_config":[{"process_username":"dbadmin","dimensions":{"service":"monitoring","component":"vertica"}}]}'

       Can specify monitoring by both process_username and process_names:
       monasca-setup -d ProcessCheck -json \
         '{"process_config":[{"process_names":["monasca-api"],"dimensions":{"service":"monitoring"}},
                             {"process_username":"mon-api","dimensions":{"service":"monitoring","component":"monasca-api"}}]}'

    """
    def __init__(self, template_dir, overwrite=False, args=None, **kwargs):
        self.process_config = []
        self.valid_process_names = []
        self.valid_usernames = []
        if 'process_config' in kwargs:
            self.process_config = kwargs['process_config']

        super(ProcessCheck, self).__init__(template_dir, overwrite, args)

    def _get_config(self):
        self.conf_file_path = None
        if self.args:
            self.conf_file_path = self.args.get('conf_file_path', None)
        if self.conf_file_path:
            self._read_config(self.conf_file_path)

    def _read_config(self, config_file):
        log.info("\tUsing parameters from config file: {}".format(config_file))
        with open(config_file) as data_file:
            try:
                data = yaml.safe_load(data_file)
                if 'process_config' in data:
                    self.process_config = data['process_config']
                else:
                    log.error("\tInvalid format yaml file, missing key: process_config")
            except yaml.YAMLError as e:
                exception_msg = ("Could not read config file. Invalid yaml format detected. {0}.".format(e))
                raise Exception(exception_msg)

    def _detect(self):
        """Run detection, set self.available True if the service is detected.

        """
        self._get_config()

        for process_item in self.process_config:
            if 'dimensions' not in process_item:
                process_item['dimensions'] = {}
            if 'process_names' in process_item:
                found_process_names = []
                not_found_process_names = []
                for process_name in process_item['process_names']:
                    if find_process_cmdline(process_name) is not None:
                        found_process_names.append(process_name)
                    else:
                        not_found_process_names.append(process_name)

                # monitoring by process_names
                if not_found_process_names:
                    log.info("\tDid not discover process_name(s): {0}.".format(",".join(not_found_process_names)))
                if found_process_names:
                    process_item['found_process_names'] = found_process_names
                    if 'exact_match' in process_item:
                        if isinstance(process_item['exact_match'], basestring):
                            process_item['exact_match'] = (process_item['exact_match'].lower() == 'true')
                    else:
                        process_item['exact_match'] = False
                    self.valid_process_names.append(process_item)

            if 'process_username' in process_item:
                if 'component' in process_item['dimensions']:
                    self.valid_usernames.append(process_item)
                else:
                    log.error("\tMissing required component dimension, when monitoring by "
                              "process_username: {}".format(process_item['process_username']))

        if self.valid_process_names or self.valid_usernames:
            self.available = True

    def _monitor_by_process_name(self, process_name, exact_match=False, detailed=True, dimensions=None):
        config = monasca_setup.agent_config.Plugins()
        instance = {'name': process_name,
                    'detailed': detailed,
                    'exact_match': exact_match,
                    'search_string': [process_name],
                    'dimensions': {}}
        # default component to process name if not given
        if dimensions:
            instance['dimensions'].update(dimensions)
            if 'component' not in dimensions:
                instance['dimensions']['component'] = process_name
        else:
            instance['dimensions']['component'] = process_name
        config['process'] = {'init_config': None, 'instances': [instance]}
        return config

    def _monitor_by_process_username(self, process_username, detailed=True, dimensions=None):
        config = monasca_setup.agent_config.Plugins()
        instance = {'name': dimensions['component'],
                    'detailed': detailed,
                    'username': process_username,
                    'dimensions': {}}
        if dimensions:
            instance['dimensions'].update(dimensions)
        config['process'] = {'init_config': None, 'instances': [instance]}
        return config

    def build_config(self):
        """Build the config as a Plugins object and return.

        """
        config = monasca_setup.agent_config.Plugins()

        # Watch by process_names
        for process in self.valid_process_names:
            log.info("\tMonitoring by process_name(s): {0} "
                     "for dimensions: {1}.".format(",".join(process['found_process_names']),
                                                   json.dumps(process['dimensions'])))
            for process_name in process['found_process_names']:
                config.merge(self._monitor_by_process_name(process_name=process_name,
                                                           dimensions=process['dimensions'],
                                                           exact_match=process['exact_match']))

        # Watch by process_username
        for process in self.valid_usernames:
            log.info("\tMonitoring by process_username: {0} "
                     "for dimensions: {1}.".format(process['process_username'], json.dumps(process['dimensions'])))
            config.merge(self._monitor_by_process_username(process_username=process['process_username'],
                                                           dimensions=process['dimensions']))
        return config

    def dependencies_installed(self):
        """Return True if dependencies are installed.

        """
        return True



# (C) Copyright 2016 Hewlett Packard Enterprise Development LP

import mock
import unittest

from monasca_setup.detection.plugins.host_alive import HostAlive

class TestHostAliveDetect(unittest.TestCase):

    def setUp(self):
        unittest.TestCase.setUp(self)
        self._host_alive = HostAlive('AAAA')
        self._expected_config = {
            'host_alive':
                {
                 'init_config':
                    {
                     'ssh_timeout': self._host_alive.DEFAULT_SSH_TIMEOUT,
                     'ping_timeout': self._host_alive.DEFAULT_PING_TIMEOUT,
                     'ssh_port': self._host_alive.DEFAULT_SSH_PORT
                    }
                }
        }

    def _create_instances(self, host_names, target_hostnames=None):
        instances = []
        index = 0
        for name in host_names:
            instance = {
                'alive_test': 'ping',
                'name': name + ' ping',
                'host_name': name}
            if (target_hostnames and
                index < len(target_hostnames)):
                target_hostname = target_hostnames[index]
                # It is possible that a target_hostname is not
                # set for each hostname
                if target_hostname:
                    instance['target_hostname'] = target_hostname
            index += 1
            instances.append(instance)
        self._expected_config['host_alive']['instances'] = instances

    def _run_build_config(self, host_names, target_hostnames=None):
        hostname = ','.join(host_names)
        args = {
            'type': 'ping',
            'hostname': hostname,
        }
        if target_hostnames:
            args['target_hostname'] = ','.join(target_hostnames)
        self._host_alive.args = args
        config = self._host_alive.build_config()
        self._create_instances(host_names, target_hostnames)
        self.assertEqual(config, self._expected_config)

    def test_build_config_simple(self):
        hostname = 'aaaa'
        self._run_build_config([hostname])

    def test_build_config_multiple(self):
        host_names = ['aaaa', 'bbbb', 'cccc']
        self._run_build_config(host_names)

    def test_build_config_complex(self):
        host_names = ['aaaa', 'bbbb', 'cccc']
        target_hostnames = ['dddd', 'eeee', 'ffff']
        self._run_build_config(host_names, target_hostnames)

    def test_build_config_complex_sparse(self):
        host_names = ['aaaa', 'bbbb', 'cccc']
        target_hostnames = ['dddd', '', 'ffff']
        self._run_build_config(host_names, target_hostnames)

    def test_build_config_complex_not_matching(self):
        host_names = ['aaaa', 'bbbb', 'cccc']
        target_hostnames = ['dddd']
        self._run_build_config(host_names, target_hostnames)

    def test_build_config_complex_too_many(self):
        host_names = ['aaaa', 'bbbb', 'cccc']
        target_hostnames = ['dddd', 'eeee', 'ffff', 'gggg']
        with self.assertRaises(Exception):
            self._run_build_config(host_names, target_hostnames)


# (C) Copyright 2015-2016 Hewlett Packard Enterprise Development LP

import collections
import copy
import json
import logging
import random
import time

import monasca_agent.common.keystone as keystone
import monascaclient.client

log = logging.getLogger(__name__)


class MonascaAPI(object):

    """Sends measurements to MonascaAPI
        Any errors should raise an exception so the transaction calling
        this is not committed
    """

    LOG_INTERVAL = 10  # messages
    MIN_BACKOFF = 10   # seconds
    MAX_BACKOFF = 60   # seconds

    def __init__(self, config):
        """Initialize Mon api client connection."""
        self.config = config
        self.url = config['url']
        self.api_version = '2_0'
        self.keystone = keystone.Keystone(config)
        self.mon_client = None
        self._failure_reason = None
        self._resume_time = None
        self._log_interval_remaining = 1
        self._current_number_measurements = 0
        self.max_buffer_size = int(config['max_buffer_size'])
        self.max_measurement_buffer_size = int(config['max_measurement_buffer_size'])

        if self.max_buffer_size > -1:
            log.debug("'max_buffer_size' is deprecated. Please use"
                      " 'max_measurement_buffer_size' instead")
            if self.max_measurement_buffer_size > -1:
                log.debug("Overriding 'max_buffer_size' option with new"
                          " 'max_measurment_buffer_size' option")
                self.max_buffer_size = -1

        self.backlog_send_rate = int(config['backlog_send_rate'])
        if self.max_buffer_size > -1:
            self.message_queue = collections.deque(maxlen=self.max_buffer_size)
        else:
            self.message_queue = collections.deque()
        self.write_timeout = int(config['write_timeout'])
        # 'amplifier' is completely optional and may not exist in the config
        try:
            self.amplifier = int(config['amplifier'])
        except KeyError:
            self.amplifier = None

    def _post(self, measurements, tenant=None):
        """Does the actual http post
            measurements is a list of Measurement
        """
        kwargs = {
            'jsonbody': measurements
        }

        if tenant:
            kwargs['tenant_id'] = tenant

        if not self.mon_client:
            self.mon_client = self.get_monclient()
            if not self.mon_client:
                # Keystone is down, queue the message
                self._queue_message(kwargs.copy(), "Keystone API is down or unreachable")
                return

        if self._send_message(**kwargs):
            if len(self.message_queue) > 0:
                messages_sent = 0
                for index in range(0, len(self.message_queue)):
                    if index < self.backlog_send_rate:

                        msg = json.loads(self.message_queue.pop())

                        if self._send_message(**msg):
                            messages_sent += 1
                            for value in msg.values():
                                self._current_number_measurements -= len(value)
                        else:
                            self._queue_message(msg, self._failure_reason)
                            break
                    else:
                        break
                log.info("Sent {0} messages from the backlog.".format(messages_sent))
                log.info("{0} messages remaining in the queue.".format(len(self.message_queue)))
                self._log_interval_remaining = 0
        else:
            self._queue_message(kwargs.copy(), self._failure_reason)

    def post_metrics(self, measurements):
        """post_metrics
            given [Measurement, ...], format the request and post to
            the monitoring api
        """
        # Add default dimensions
        for envelope in measurements:
            measurement = envelope['measurement']
            if isinstance(measurement['dimensions'], list):
                measurement['dimensions'] = dict([(d[0], d[1]) for d in measurement['dimensions']])

        # Split out separate POSTs for each delegated tenant (includes 'None')
        tenant_group = {}
        for envelope in measurements:
            measurement = envelope['measurement']
            tenant = envelope['tenant_id']
            tenant_group.setdefault(tenant, []).append(copy.deepcopy(measurement))

        for tenant in tenant_group:
            self._post(tenant_group[tenant], tenant)

    def get_monclient(self):
        """get_monclient
            get a new monasca-client object
        """
        token = self.keystone.get_token()
        if token:
            # Create the client.
            kwargs = self.keystone.get_credential_args()
            kwargs['token'] = token
            if not self.url:
                self.url = self.keystone.get_monasca_url()

            return monascaclient.client.Client(self.api_version, self.url, write_timeout=self.write_timeout, **kwargs)

        return None

    def _send_message(self, **kwargs):
        if self._resume_time:
            if time.time() > self._resume_time:
                self._resume_time = None
                log.debug("Getting new token...")
                # Get a new keystone client and token
                if self.keystone.refresh_token():
                    self.mon_client.replace_token(self.keystone.get_token())
            else:
                # Return without posting so the monasca client doesn't keep requesting new tokens
                return False
        try:
            self.mon_client.metrics.create(**kwargs)
            return True
        except monascaclient.exc.HTTPException as ex:
            if ex.code == 401:
                # monasca client should already have retried once with a new token before returning this exception
                self._failure_reason = 'Invalid token detected. Waiting to get new token from Keystone'
                wait_time = random.randint(MonascaAPI.MIN_BACKOFF, MonascaAPI.MAX_BACKOFF + 1)
                self._resume_time = time.time() + wait_time
                log.info("Invalid token detected. Waiting %d seconds before getting new token.", wait_time)
            else:
                log.exception("HTTPException: error sending message to monasca-api.")
                self._failure_reason = 'Error sending message to the Monasca API: {0}'.format(str(ex.message))
        except Exception:
            log.exception("Error sending message to Monasca API.")
            self._failure_reason = 'The Monasca API is DOWN or unreachable'

        return False

    def _queue_message(self, msg, reason):
        if self.max_buffer_size == 0 or self.max_measurement_buffer_size == 0:
            return

        self.message_queue.append(json.dumps(msg))

        for value in msg.values():
            self._current_number_measurements += len(value)

        if self.max_measurement_buffer_size > -1:
            while self._current_number_measurements > self.max_measurement_buffer_size:
                self._remove_oldest_from_queue()

        if self._log_interval_remaining <= 1:
            log.warn("{0}. Queuing the messages to send later...".format(reason))
            log.info("Current agent queue size: {0} of {1}.".format(len(self.message_queue),
                                                                    self.max_buffer_size))
            log.info("Current measurements in queue: {0} of {1}".format(
                self._current_number_measurements, self.max_measurement_buffer_size))

            log.info("A message will be logged for every {0} messages queued.".format(MonascaAPI.LOG_INTERVAL))
            self._log_interval_remaining = MonascaAPI.LOG_INTERVAL
        else:
            self._log_interval_remaining -= 1

    def _remove_oldest_from_queue(self):
        removed_batch = json.loads(self.message_queue.popleft())
        num_discarded = 0
        for value in removed_batch.values():
            num_discarded += len(value)
        self._current_number_measurements -= num_discarded
        log.warn("Queue too large, discarding oldest batch: {0} measurements discarded".format(
            num_discarded))



# (C) Copyright 2015-2016 Hewlett Packard Enterprise Development Company LP

import ast
import logging

import monasca_setup.agent_config
import monasca_setup.detection

log = logging.getLogger(__name__)


class HttpCheck(monasca_setup.detection.ArgsPlugin):
    """Setup an http_check according to the passed in args.
       Despite being a detection plugin this plugin does no detection and will be a noop without arguments.
       Expects space separated arguments, the required argument is url. Optional parameters include:
       disable_ssl_validation and match_pattern.
    """

    def _detect(self):
        """Run detection, set self.available True if the service is detected.
        """
        self.available = self._check_required_args(['url'])

    def build_config(self):
        """Build the config as a Plugins object and return.
        """
        config = monasca_setup.agent_config.Plugins()
        # No support for setting headers at this time
        instance = self._build_instance(['url', 'timeout', 'username', 'password',
                                         'match_pattern', 'disable_ssl_validation',
                                         'name', 'use_keystone', 'collect_response_time'])

        # Normalize any boolean parameters
        for param in ['use_keystone', 'collect_response_time']:
            if param in self.args:
                instance[param] = ast.literal_eval(self.args[param].capitalize())
        # Set some defaults
        if 'collect_response_time' not in instance:
            instance['collect_response_time'] = True
        if 'name' not in instance:
            instance['name'] = self.args['url']

        config['http_check'] = {'init_config': None, 'instances': [instance]}

        return config



# (C) Copyright 2016 Hewlett Packard Enterprise Development LP

import contextlib
import logging
import os
import psutil
import tempfile
import unittest

from mock import patch

from monasca_setup.detection.plugins.process import ProcessCheck

LOG = logging.getLogger('monasca_setup.detection.plugins.process')


class PSUtilGetProc(object):
    cmdLine = ['monasca-api']

    def as_dict(self, attrs=None):
        return {'name': 'monasca-api',
                'cmdline': PSUtilGetProc.cmdLine}

    def cmdline(self):
        return self.cmdLine


class TestProcessCheck(unittest.TestCase):

    def setUp(self):
        unittest.TestCase.setUp(self)
        with patch.object(ProcessCheck, '_detect') as mock_detect:
            self.proc_plugin = ProcessCheck('temp_dir')
            self.assertTrue(mock_detect.called)

    def _detect(self,
                proc_plugin,
                config_is_file=False,
                by_process_name=True):
        proc_plugin.available = False
        psutil_mock = PSUtilGetProc()

        process_iter_patch = patch.object(psutil, 'process_iter',
                                          return_value=[psutil_mock])
        isfile_patch = patch.object(os.path, 'isfile',
                                    return_value=config_is_file)

        with contextlib.nested(process_iter_patch,
                               isfile_patch) as (
                mock_process_iter, mock_isfile):
            proc_plugin._detect()
            if by_process_name:
                self.assertTrue(mock_process_iter.called)
            self.assertFalse(mock_isfile.called)

    def test_detect_process_not_found(self):
        PSUtilGetProc.cmdLine = []
        self.proc_plugin.process_config = [{'process_names': ['monasca-api'], 'dimensions': {'service': 'monitoring'}}]
        self._detect(self.proc_plugin)
        self.assertFalse(self.proc_plugin.available)

    def test_detect_process_found(self):
        self.proc_plugin.process_config = [{'process_names': ['monasca-api'], 'dimensions': {'service': 'monitoring'}}]
        self._detect(self.proc_plugin)
        self.assertTrue(self.proc_plugin.available)

    def test_missing_arg(self):
        # monitor by process_username requires component
        self.proc_plugin.process_config = [{'process_username': 'dbadmin', 'dimensions': {'service': 'monitoring'}}]
        self._detect(self.proc_plugin, by_process_name=False)
        self.assertFalse(self.proc_plugin.available)

    def test_detect_build_config_process_name(self):
        self.proc_plugin.process_config = [{'process_names': ['monasca-api'], 'dimensions': {'service': 'monitoring'}}]
        self._detect(self.proc_plugin)
        result = self.proc_plugin.build_config()
        self.assertEqual(result['process']['instances'][0]['name'],
                         'monasca-api')
        self.assertEqual(result['process']['instances'][0]['detailed'],
                         True)
        self.assertEqual(result['process']['instances'][0]['exact_match'],
                         False)
        self.assertEqual(result['process']['instances'][0]['dimensions']['service'],
                         'monitoring')
        self.assertEqual(result['process']['instances'][0]['dimensions']['component'],
                         'monasca-api')
        self.assertEqual(result['process']['instances'][0]['search_string'][0],
                         'monasca-api')

    def test_detect_build_config_process_name_exact_match_true(self):
        self.proc_plugin.process_config = [
            {'process_names': ['monasca-api'], 'dimensions': {'service': 'monitoring'}, 'exact_match': True}]
        self._detect(self.proc_plugin)
        result = self.proc_plugin.build_config()
        self.assertEqual(result['process']['instances'][0]['name'],
                         'monasca-api')
        self.assertEqual(result['process']['instances'][0]['detailed'],
                         True)
        self.assertEqual(result['process']['instances'][0]['exact_match'],
                         True)
        self.assertEqual(result['process']['instances'][0]['dimensions']['service'],
                         'monitoring')
        self.assertEqual(result['process']['instances'][0]['dimensions']['component'],
                         'monasca-api')
        self.assertEqual(result['process']['instances'][0]['search_string'][0],
                         'monasca-api')

    def test_build_config_process_names(self):
        self.proc_plugin.valid_process_names = [
            {'process_names': ['monasca-api'],
             'dimensions': {'service': 'monitoring'},
             'found_process_names': ['monasca-api'],
             'exact_match': False},
            {'process_names': ['monasca-thresh'],
             'dimensions': {'service': 'monitoring'},
             'found_process_names': ['monasca-thresh'],
             'exact_match': False}]
        result = self.proc_plugin.build_config()
        self.assertEqual(result['process']['instances'][0]['name'],
                         'monasca-api')
        self.assertEqual(result['process']['instances'][0]['detailed'],
                         True)
        self.assertEqual(result['process']['instances'][0]['exact_match'],
                         False)
        self.assertEqual(result['process']['instances'][0]['dimensions']['service'],
                         'monitoring')
        self.assertEqual(result['process']['instances'][0]['dimensions']['component'],
                         'monasca-api')
        self.assertEqual(result['process']['instances'][0]['search_string'][0],
                         'monasca-api')
        self.assertEqual(result['process']['instances'][1]['name'],
                         'monasca-thresh')
        self.assertEqual(result['process']['instances'][1]['dimensions']['component'],
                         'monasca-thresh')

    def test_detect_build_config_process_username(self):
        self.proc_plugin.process_config = \
            [{'process_username': 'dbadmin', 'dimensions': {'service': 'monitoring', 'component': 'vertica'}}]
        self.proc_plugin._detect()
        result = self.proc_plugin.build_config()
        self.assertEqual(result['process']['instances'][0]['name'],
                         'vertica')
        self.assertEqual(result['process']['instances'][0]['detailed'],
                         True)
        self.assertEqual(result['process']['instances'][0]['dimensions']['service'],
                         'monitoring')
        self.assertEqual(result['process']['instances'][0]['dimensions']['component'],
                         'vertica')

    def test_input_yaml_file(self):
        # note: The previous tests will cover all yaml data variations, since the data is translated into a single dictionary.
        fd, temp_path = tempfile.mkstemp(suffix='.yaml')
        os.write(fd, '---\nprocess_config:\n- process_username: dbadmin\n  dimensions:\n    '
                 'service: monitoring\n    component: vertica\n')
        self.proc_plugin.args = {'conf_file_path': temp_path}
        self.proc_plugin._detect()
        result = self.proc_plugin.build_config()
        self.assertEqual(result['process']['instances'][0]['name'],
                         'vertica')
        self.assertEqual(result['process']['instances'][0]['detailed'],
                         True)
        self.assertEqual(result['process']['instances'][0]['dimensions']['service'],
                         'monitoring')
        self.assertEqual(result['process']['instances'][0]['dimensions']['component'],
                         'vertica')
        os.close(fd)
        os.remove(temp_path)



# (C) Copyright 2016 Hewlett Packard Enterprise Development Company LP

from os.path import abspath
from os.path import exists
from os.path import isfile
from os.path import join

from os import listdir
from os import stat
from os import walk

import logging
import monasca_agent.collector.checks as checks

log = logging.getLogger(__name__)


class FileSize(checks.AgentCheck):

    """This check is for monitoring and reporting metrics on the size of
    files for a provided directory

    Config options:
        "directory_name" - string, directory of the file (required)
        "file_names" - list of strings, file names under directory_name to
                       gather stats for (required)
        "recursive" - boolean, when true and file_name is set to '*' will
                      recursively grab files under the given directory to
                      gather stats on.
    """
    def __init__(self, name, init_config, agent_config, instances=None):
        super(FileSize, self).__init__(name, init_config, agent_config,
                                       instances)

    def check(self, instance):
        if "directory_name" not in instance:
            raise Exception('FileSize Check: missing "directory_name" in '
                            'config')
        if "file_names" not in instance:
            raise Exception('FileSize Check: missing "file_names" in config')
        recursive = instance.get("recursive") or False
        directory_name = instance["directory_name"]
        abs_directory = abspath(directory_name)
        if exists(abs_directory):
            if instance["file_names"] != ['*']:
                file_names = instance["file_names"]
                self._get_stats(abs_directory, file_names, instance)
            else:
                if recursive:
                    for root, dirs, files in walk(abs_directory):
                        self._get_stats(root, files, instance)
                else:
                    files = [file_name for file_name in listdir(abs_directory)
                             if isfile(join(abs_directory, file_name))]
                    self._get_stats(abs_directory, files, instance)
        else:
            log.error('FileSize Check: directory {0} does not exist'.
                      format(abs_directory))

    def _get_stats(self, directory_name, files, instance):
        num_files = 0
        for file_name in files:
            abs_dir_name = abspath(join(directory_name, file_name))
            if isfile(abs_dir_name):
                dimensions = self._set_dimensions({
                    "file_name": file_name,
                    "directory_name": directory_name}, instance)
                got_stats = False
                file_abs_path = join(directory_name, file_name)
                try:
                    file_stat = stat(file_abs_path)
                except OSError as ose:
                    log.warn("FileSize Check: could not stat file %s - %s" % (
                        file_abs_path, ose))
                else:
                    file_bytes = file_stat.st_size
                    self.gauge("file.size_bytes", file_bytes,
                               dimensions=dimensions)
                    got_stats = True
                if got_stats:
                    num_files += 1
            else:
                log.error('FileSize Check: file {0} does not exist'.
                          format(abs_dir_name))
        log.debug('Collected {0} file_size metrics from {1}'.
                  format(num_files, directory_name))



# (C) Copyright 2015,2016 Hewlett Packard Enterprise Development Company LP

import monasca_setup.detection


class Neutron(monasca_setup.detection.ServicePlugin):

    """Detect Neutron daemons and setup configuration to monitor them.

    """

    def __init__(self, template_dir, overwrite=True, args=None):
        service_params = {
            'args': args,
            'template_dir': template_dir,
            'overwrite': overwrite,
            'service_name': 'networking',
            'process_names': ['neutron-server', 'neutron-openvswitch-agent',
                              'neutron-rootwrap', 'neutron-dhcp-agent',
                              'neutron-vpn-agent', 'neutron-metadata-agent',
                              'neutron-metering-agent', 'neutron-l3-agent',
                              'bin/neutron-lbaas-agent',
                              'neutron-lbaasv2-agent',
                              'neutron-l2gateway-agent',
                              'infoblox-ipam-agent',
                              'ipsec/charon'],
            'service_api_url': 'http://localhost:9696',
            'search_pattern': '.*v2.0.*'
        }

        super(Neutron, self).__init__(service_params)

    def build_config(self):
        """Build the config as a Plugins object and return."""
        # Skip the http check if neutron-server is not on this box
        if 'neutron-server' not in self.found_processes:
            self.service_api_url = None
            self.search_pattern = None

        return monasca_setup.detection.ServicePlugin.build_config(self)



#!/bin/env python

# (c) Copyright 2014-2016 Hewlett Packard Enterprise Development LP
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""Monasca Agent interface for libvirt metrics"""

import json
import libvirt
import math
import monasca_agent.collector.checks.utils as utils
import os
import stat
import subprocess
import time

from calendar import timegm
from copy import deepcopy
from datetime import datetime
from datetime import timedelta
from monasca_agent.collector.checks import AgentCheck
from monasca_agent.collector.virt import inspector
from multiprocessing.dummy import Pool
from netaddr import all_matching_cidrs

DOM_STATES = {libvirt.VIR_DOMAIN_BLOCKED: 'VM is blocked',
              libvirt.VIR_DOMAIN_CRASHED: 'VM has crashed',
              libvirt.VIR_DOMAIN_NONE: 'VM has no state',
              libvirt.VIR_DOMAIN_PAUSED: 'VM is paused',
              libvirt.VIR_DOMAIN_PMSUSPENDED: 'VM is in power management (s3) suspend',
              libvirt.VIR_DOMAIN_SHUTDOWN: 'VM is shutting down',
              libvirt.VIR_DOMAIN_SHUTOFF: 'VM has been shut off (other reason)'}
DOM_SHUTOFF_STATES = {libvirt.VIR_DOMAIN_SHUTOFF_UNKNOWN: 'VM has been shutoff (reason unknown)',
                      libvirt.VIR_DOMAIN_SHUTOFF_SHUTDOWN: 'VM has been shut down',
                      libvirt.VIR_DOMAIN_SHUTOFF_DESTROYED: 'VM has been destroyed (forced off)',
                      libvirt.VIR_DOMAIN_SHUTOFF_CRASHED: 'VM has crashed',
                      libvirt.VIR_DOMAIN_SHUTOFF_MIGRATED: 'VM has been migrated',
                      libvirt.VIR_DOMAIN_SHUTOFF_SAVED: 'VM has been suspended',
                      libvirt.VIR_DOMAIN_SHUTOFF_FAILED: 'VM has failed to start',
                      libvirt.VIR_DOMAIN_SHUTOFF_FROM_SNAPSHOT: 'VM has been restored from powered off snapshot'}


class LibvirtCheck(AgentCheck):

    """Inherit Agent class and gather libvirt metrics"""

    def __init__(self, name, init_config, agent_config, instances=None):
        AgentCheck.__init__(self, name, init_config, agent_config, instances=[{}])
        self.instance_cache_file = "{0}/{1}".format(self.init_config.get('cache_dir'),
                                                    'libvirt_instances.json')
        self.metric_cache_file = "{0}/{1}".format(self.init_config.get('cache_dir'),
                                                  'libvirt_metrics.json')
        self.use_bits = self.init_config.get('network_use_bits')

        self._collect_intervals = {}

        self._set_collection_intervals('disk', 'disk_collection_period')
        self._set_collection_intervals('vnic', 'vnic_collection_period')

        pool_size = self.init_config.get('max_ping_concurrency', 8)
        self.pool = Pool(pool_size)

    def _set_collection_intervals(self, interval_name, config_name):
        self._collect_intervals[interval_name] = {
            'period': int(self.init_config.get(config_name, 0)),
            'last_collect': datetime.fromordinal(1),
            'skip': False}

    def _test_vm_probation(self, created):
        """Test to see if a VM was created within the probation period.

        Convert an ISO-8601 timestamp into UNIX epoch timestamp from now
        and compare that against configured vm_probation.  Return the
        number of seconds this VM will remain in probation.
        """
        dt = datetime.strptime(created, '%Y-%m-%dT%H:%M:%SZ')
        created_sec = (time.time() - timegm(dt.timetuple()))
        probation_time = self.init_config.get('vm_probation', 300) - created_sec
        return int(probation_time)

    def _get_metric_name(self, orig_name):
        # Rename "tx" to "out" and "rx" to "in"
        metric_name = orig_name.replace("tx", "out").replace("rx", "in")
        if self.use_bits:
            metric_name = metric_name.replace("bytes", "bits")
        return metric_name

    @staticmethod
    def _get_metric_rate_name(metric_name):
        """Change the metric name to a rate, i.e. "net.rx_bytes"
        gets converted to "net.rx_bytes_sec"
        """
        return "{0}_sec".format(metric_name)

    @staticmethod
    def _validate_secgroup(cache, instance, source_ip):
        """Search through an instance's security groups for pingability
        """
        for instance_secgroup in instance.security_groups:
            for secgroup in cache:
                if ((secgroup['tenant_id'] == instance.tenant_id and
                     secgroup['name'] == instance_secgroup['name'])):
                    for rule in secgroup['security_group_rules']:
                        if rule['protocol'] == 'icmp':
                            if ((not rule['remote_ip_prefix'] or
                                 all_matching_cidrs(source_ip,
                                                    [rule['remote_ip_prefix']]))):
                                return True

    def _update_instance_cache(self):
        """Collect instance_id, project_id, and AZ for all instance UUIDs
        """
        from novaclient import client
        from novaclient.exceptions import NotFound

        id_cache = {}
        flavor_cache = {}
        port_cache = None
        netns = None
        # Get a list of all instances from the Nova API
        nova_client = client.Client(2,
                                    username=self.init_config.get('admin_user'),
                                    password=self.init_config.get('admin_password'),
                                    project_name=self.init_config.get('admin_tenant_name'),
                                    auth_url=self.init_config.get('identity_uri'),
                                    endpoint_type='internalURL',
                                    service_type="compute",
                                    region_name=self.init_config.get('region_name'))
        instances = nova_client.servers.list(search_opts={'all_tenants': 1,
                                                          'host': self.hostname})
        # Lay the groundwork for fetching VM IPs and network namespaces
        if self.init_config.get('ping_check'):
            from neutronclient.v2_0 import client
            nu = client.Client(username=self.init_config.get('admin_user'),
                               password=self.init_config.get('admin_password'),
                               tenant_name=self.init_config.get('admin_tenant_name'),
                               auth_url=self.init_config.get('identity_uri'),
                               endpoint_type='internalURL',
                               region_name=self.init_config.get('region_name'))
            port_cache = nu.list_ports()['ports']
            # Finding existing network namespaces is an indication that either
            # DVR agent_mode is enabled, or this is all-in-one (like devstack)
            netns = subprocess.check_output(['ip', 'netns', 'list'])
            if netns == '':
                self.log.warn("Unable to ping VMs, no network namespaces found." +
                              "Either no VMs are present, or routing is centralized.")

        #
        # Only make the keystone call to get the tenant list
        # if we are configured to publish tenant names.
        #
        tenants = []
        if self.init_config.get('metadata') and 'tenant_name' in self.init_config.get('metadata'):
            tenants = utils.get_tenant_list(self.init_config, self.log)

        for instance in instances:
            instance_ports = []
            inst_name = instance.__getattr__('OS-EXT-SRV-ATTR:instance_name')
            inst_az = instance.__getattr__('OS-EXT-AZ:availability_zone')
            if instance.flavor['id'] in flavor_cache:
                inst_flavor = flavor_cache[instance.flavor['id']]
            else:
                try:
                    inst_flavor = nova_client.flavors.get(instance.flavor['id'])
                except NotFound as e:
                    self.log.error('Skipping VM {}: {}'.format(inst_name, e))
                    continue
                flavor_cache[instance.flavor['id']] = inst_flavor
            if port_cache:
                instance_ports = [p['id'] for p in port_cache if p['device_id'] == instance.id]
            id_cache[inst_name] = {'instance_uuid': instance.id,
                                   'hostname': instance.name,
                                   'zone': inst_az,
                                   'created': instance.created,
                                   'tenant_id': instance.tenant_id,
                                   'vcpus': inst_flavor.vcpus,
                                   'ram': inst_flavor.ram,
                                   'disk': inst_flavor.disk,
                                   'instance_ports': instance_ports}

            tenant_name = utils.get_tenant_name(tenants, instance.tenant_id)
            if tenant_name:
                id_cache[inst_name]['tenant_name'] = tenant_name

            for config_var in ['metadata', 'customer_metadata']:
                if self.init_config.get(config_var):
                    for metadata in self.init_config.get(config_var):
                        if instance.metadata.get(metadata):
                            id_cache[inst_name][metadata] = (instance.metadata.
                                                             get(metadata))

            # Build a list of pingable IP addresses attached to this VM and the
            # appropriate namespace, for use in ping tests
            if netns:
                secgroup_cache = nu.list_security_groups()['security_groups']
                self._build_ip_list(instance, inst_name,
                                    secgroup_cache, port_cache, id_cache)

        id_cache['last_update'] = int(time.time())

        # Write the updated cache
        try:
            with open(self.instance_cache_file, 'w') as cache_json:
                json.dump(id_cache, cache_json)
            if stat.S_IMODE(os.stat(self.instance_cache_file).st_mode) != 0o600:
                os.chmod(self.instance_cache_file, 0o600)
        except IOError as e:
            self.log.error("Cannot write to {0}: {1}".format(self.instance_cache_file, e))

        return id_cache

    def _build_ip_list(self, instance, inst_name, secgroup_cache, port_cache, id_cache):
        # Find all active fixed IPs for this VM, fetch each subnet_id
        for net in instance.addresses:
            for ip in instance.addresses[net]:
                if ip['OS-EXT-IPS:type'] == 'fixed' and ip['version'] == 4:
                    subnet_id = None
                    nsuuid = None
                    for port in port_cache:
                        if ((port['mac_address'] == ip['OS-EXT-IPS-MAC:mac_addr'] and
                             port['tenant_id'] == instance.tenant_id and
                             port['status'] == 'ACTIVE')):
                            for fixed in port['fixed_ips']:
                                if fixed['ip_address'] == ip['addr']:
                                    subnet_id = fixed['subnet_id']
                                    break
                    # Use the subnet_id to find the router
                    ping_allowed = False
                    if subnet_id is not None:
                        for port in port_cache:
                            if ((port['device_owner'].startswith('network:router_interface') and
                                 port['tenant_id'] == instance.tenant_id and
                                 port['status'] == 'ACTIVE')):
                                nsuuid = port['device_id']
                                for fixed in port['fixed_ips']:
                                    if fixed['subnet_id'] == subnet_id:
                                        # Validate security group
                                        if self._validate_secgroup(secgroup_cache,
                                                                   instance,
                                                                   fixed['ip_address']):
                                            ping_allowed = True
                                            break
                            if nsuuid is not None:
                                break
                    if nsuuid is not None and ping_allowed:
                        if 'network' not in id_cache[inst_name]:
                            id_cache[inst_name]['network'] = []
                        id_cache[inst_name]['network'].append({'namespace': "qrouter-{0}".format(nsuuid),
                                                               'ip': ip['addr']})
                    elif ping_allowed is False:
                        self.log.debug("ICMP disallowed for {0} on {1}".format(inst_name,
                                                                               ip['addr']))

    def _load_instance_cache(self):
        """Load the cache map of instance names to Nova data.
           If the cache does not yet exist or is damaged, (re-)build it.
        """
        instance_cache = {}
        try:
            with open(self.instance_cache_file, 'r') as cache_json:
                instance_cache = json.load(cache_json)

                # Is it time to force a refresh of this data?
                if self.init_config.get('nova_refresh') is not None:
                    time_diff = time.time() - instance_cache['last_update']
                    if time_diff > self.init_config.get('nova_refresh'):
                        self._update_instance_cache()
        except (IOError, TypeError, ValueError):
            # The file may not exist yet, or is corrupt.  Rebuild it now.
            self.log.warning("Instance cache missing or corrupt, rebuilding.")
            instance_cache = self._update_instance_cache()
            pass

        return instance_cache

    def _load_metric_cache(self):
        """Load the counter metrics from the previous collection iteration
        """
        metric_cache = {}
        try:
            with open(self.metric_cache_file, 'r') as cache_json:
                metric_cache = json.load(cache_json)
        except (IOError, TypeError, ValueError):
            # The file may not exist yet.
            self.log.warning("Metrics cache missing or corrupt, rebuilding.")
            metric_cache = {}
            pass

        return metric_cache

    def _update_metric_cache(self, metric_cache, run_time):
        # Remove inactive VMs from the metric cache
        write_metric_cache = deepcopy(metric_cache)
        for instance in metric_cache:
            if (('cpu.time' not in metric_cache[instance] or
                 self._test_vm_probation(time.strftime('%Y-%m-%dT%H:%M:%SZ',
                                         time.gmtime(metric_cache[instance]['cpu.time']['timestamp'] + run_time))) < 0)):
                self.log.info("Expiring old/empty {0} from cache".format(instance))
                del(write_metric_cache[instance])
        try:
            with open(self.metric_cache_file, 'w') as cache_json:
                json.dump(write_metric_cache, cache_json)
            if stat.S_IMODE(os.stat(self.metric_cache_file).st_mode) != 0o600:
                os.chmod(self.metric_cache_file, 0o600)
        except IOError as e:
            self.log.error("Cannot write to {0}: {1}".format(self.metric_cache_file, e))

    def _inspect_network(self, insp, inst, inst_name, instance_cache, metric_cache, dims_customer, dims_operations):
        """Inspect network metrics for an instance"""
        for vnic in insp.inspect_vnics(inst):
            sample_time = time.time()
            vnic_dimensions = {'device': vnic[0].name}
            instance_ports = instance_cache.get(inst_name)['instance_ports']
            partial_port_id = vnic[0].name.split('tap')[1]
            # Multiple networked guest
            for port in instance_ports:
                if partial_port_id == port[:11]:
                    vnic_dimensions['port_id'] = port
                    break
            for metric in vnic[1]._fields:
                metric_name = "net.{0}".format(metric)
                if metric_name not in metric_cache[inst_name]:
                    metric_cache[inst_name][metric_name] = {}

                value = int(vnic[1].__getattribute__(metric))
                if vnic[0].name in metric_cache[inst_name][metric_name]:
                    last_update_time = metric_cache[inst_name][metric_name][vnic[0].name]['timestamp']
                    time_diff = sample_time - float(last_update_time)
                    rate_value = self._calculate_rate(value,
                                                      metric_cache[inst_name][metric_name][vnic[0].name]['value'],
                                                      time_diff)
                    if rate_value < 0:
                        # Bad value, save current reading and skip
                        self.log.warn("Ignoring negative network sample for: "
                                      "{0} new value: {1} old value: {2}"
                                      .format(inst_name, value,
                                              metric_cache[inst_name][metric_name][vnic[0].name]['value']))
                        metric_cache[inst_name][metric_name][vnic[0].name] = {
                            'timestamp': sample_time,
                            'value': value}
                        continue
                    rate_name = self._get_metric_rate_name(metric_name)
                    rate_name = self._get_metric_name(rate_name)
                    if self.use_bits:
                        rate_value *= 8
                    # Customer
                    this_dimensions = vnic_dimensions.copy()
                    this_dimensions.update(dims_customer)
                    self.gauge(rate_name, rate_value,
                               dimensions=this_dimensions,
                               delegated_tenant=instance_cache.get(inst_name)['tenant_id'],
                               hostname=instance_cache.get(inst_name)['hostname'])
                    # Operations (metric name prefixed with "vm."
                    this_dimensions = vnic_dimensions.copy()
                    this_dimensions.update(dims_operations)
                    self.gauge("vm.{0}".format(rate_name), rate_value,
                               dimensions=this_dimensions)
                # Report raw counters.
                mapped_name = self._get_metric_name(metric_name)
                weighted_value = value
                if self.use_bits:
                    weighted_value = value * 8
                # Customer
                this_dimensions = vnic_dimensions.copy()
                this_dimensions.update(dims_customer)
                self.gauge(mapped_name, weighted_value,
                           dimensions=this_dimensions,
                           delegated_tenant=instance_cache.get(inst_name)['tenant_id'],
                           hostname=instance_cache.get(inst_name)['hostname'])
                # Operations (metric name prefixed with "vm.")
                this_dimensions = vnic_dimensions.copy()
                this_dimensions.update(dims_operations)
                self.gauge("vm.{0}".format(mapped_name),
                           weighted_value, dimensions=this_dimensions)
                # Save this metric to the cache
                metric_cache[inst_name][metric_name][vnic[0].name] = {
                    'timestamp': sample_time,
                    'value': value}

    def _inspect_cpu(self, insp, inst, inst_name, instance_cache, metric_cache, dims_customer, dims_operations):
        """Inspect cpu metrics for an instance"""

        sample_time = float("{:9f}".format(time.time()))
        cpu_info = insp.inspect_cpus(inst)

        if 'cpu.time' in metric_cache[inst_name]:
            # I have a prior value, so calculate the raw_perc & push the metric
            cpu_diff = cpu_info.time - metric_cache[inst_name]['cpu.time']['value']
            time_diff = sample_time - float(metric_cache[inst_name]['cpu.time']['timestamp'])
            # Convert time_diff to nanoseconds, and calculate percentage
            raw_perc = (cpu_diff / (time_diff * 1000000000)) * 100
            # Divide by the number of cores to normalize the percentage
            normalized_perc = (raw_perc / cpu_info.number)
            if raw_perc < 0:
                # Bad value, save current reading and skip
                self.log.warn("Ignoring negative CPU sample for: "
                              "{0} new cpu time: {1} old cpu time: {2}"
                              .format(inst_name, cpu_info.time,
                                      metric_cache[inst_name]['cpu.time']['value']))
                metric_cache[inst_name]['cpu.time'] = {'timestamp': sample_time,
                                                       'value': cpu_info.time}
                return

            self.gauge('cpu.utilization_perc', int(round(raw_perc, 0)),
                       dimensions=dims_customer,
                       delegated_tenant=instance_cache.get(inst_name)['tenant_id'],
                       hostname=instance_cache.get(inst_name)['hostname'])
            self.gauge('cpu.utilization_norm_perc', int(round(normalized_perc, 0)),
                       dimensions=dims_customer,
                       delegated_tenant=instance_cache.get(inst_name)['tenant_id'],
                       hostname=instance_cache.get(inst_name)['hostname'])
            self.gauge('vm.cpu.utilization_perc', int(round(raw_perc, 0)),
                       dimensions=dims_operations)
            self.gauge('vm.cpu.utilization_norm_perc', int(round(normalized_perc, 0)),
                       dimensions=dims_operations)

            cpu_time_name = 'cpu.time_ns'
            # cpu.time_ns for owning tenant
            self.gauge(cpu_time_name, cpu_info.time,
                       dimensions=dims_customer,
                       delegated_tenant=instance_cache.get(inst_name)['tenant_id'],
                       hostname=instance_cache.get(inst_name)['hostname'])
            # vm..cpu.time_ns for operations tenant
            self.gauge("vm.{0}".format(cpu_time_name), cpu_info.time,
                       dimensions=dims_operations)
        metric_cache[inst_name]['cpu.time'] = {'timestamp': sample_time,
                                               'value': cpu_info.time}

    def _inspect_disks(self, insp, inst, inst_name, instance_cache, metric_cache, dims_customer, dims_operations):
        """Inspect disk metrics for an instance"""

        metric_aggregate = {}
        for disk in insp.inspect_disks(inst):
            sample_time = time.time()
            disk_dimensions = {'device': disk[0].device}
            for metric in disk[1]._fields:
                metric_name = "io.{0}".format(metric.replace('requests', 'ops'))
                if metric_name not in metric_cache[inst_name]:
                    metric_cache[inst_name][metric_name] = {}

                value = int(disk[1].__getattribute__(metric))
                metric_aggregate[metric_name] = metric_aggregate.get(
                    metric_name, 0) + value
                if disk[0].device in metric_cache[inst_name][metric_name]:
                    cached_val = metric_cache[inst_name][metric_name][disk[
                        0].device]['value']
                    last_update_time = metric_cache[inst_name][metric_name][disk[
                        0].device]['timestamp']
                    time_diff = sample_time - float(last_update_time)
                    rate_value = self._calculate_rate(value, cached_val, time_diff)
                    if rate_value < 0:
                        # Bad value, save current reading and skip
                        self.log.warn("Ignoring negative disk sample for: "
                                      "{0} new value: {1} old value: {2}"
                                      .format(inst_name, value, cached_val))
                        metric_cache[inst_name][metric_name][disk[0].device] = {
                            'timestamp': sample_time,
                            'value': value}
                        continue
                    # Change the metric name to a rate, ie. "io.read_requests"
                    # gets converted to "io.read_ops_sec"
                    rate_name = "{0}_sec".format(metric_name.replace('requests', 'ops'))
                    # Customer
                    this_dimensions = disk_dimensions.copy()
                    this_dimensions.update(dims_customer)
                    self.gauge(rate_name, rate_value, dimensions=this_dimensions,
                               delegated_tenant=instance_cache.get(inst_name)['tenant_id'],
                               hostname=instance_cache.get(inst_name)['hostname'])
                    self.gauge(metric_name, value, dimensions=this_dimensions,
                               delegated_tenant=instance_cache.get(inst_name)['tenant_id'],
                               hostname=instance_cache.get(inst_name)['hostname'])

                    # Operations (metric name prefixed with "vm."
                    this_dimensions = disk_dimensions.copy()
                    this_dimensions.update(dims_operations)
                    self.gauge("vm.{0}".format(rate_name), rate_value,
                               dimensions=this_dimensions)
                    self.gauge("vm.{0}".format(metric_name), value,
                               dimensions=this_dimensions)
                # Save this metric to the cache
                metric_cache[inst_name][metric_name][disk[0].device] = {
                    'timestamp': sample_time,
                    'value': value}

        if self.init_config.get('vm_extended_disks_check_enable'):
            for metric in metric_aggregate:
                sample_time = time.time()
                rate_name = "{0}_total_sec".format(metric)
                if rate_name not in metric_cache[inst_name]:
                    metric_cache[inst_name][rate_name] = {}
                else:
                    last_update_time = metric_cache[inst_name][
                        rate_name]['timestamp']
                    time_diff = sample_time - float(last_update_time)
                    rate_value = self._calculate_rate(metric_aggregate[metric],
                                                      metric_cache[inst_name][rate_name]['value'],
                                                      time_diff)
                    if rate_value < 0:
                        # Bad value, save current reading and skip
                        self.log.warn("Ignoring negative disk sample for: "
                                      "{0} new value: {1} old value: {2}"
                                      .format(inst_name, metric_aggregate[metric],
                                              metric_cache[inst_name][rate_name][
                                                  'value']))
                        metric_cache[inst_name][rate_name] = {
                            'timestamp': sample_time,
                            'value': metric_aggregate[metric]}
                        continue
                    self.gauge(rate_name, rate_value, dimensions=dims_customer,
                               delegated_tenant=instance_cache.get(inst_name)['tenant_id'],
                               hostname=instance_cache.get(inst_name)['hostname'])
                    self.gauge("vm.{0}".format(rate_name), rate_value,
                               dimensions=dims_operations)
                self.gauge("{0}_total".format(metric), metric_aggregate[metric],
                           dimensions=dims_customer,
                           delegated_tenant=instance_cache.get(inst_name)['tenant_id'],
                           hostname=instance_cache.get(inst_name)['hostname'])
                self.gauge("vm.{0}_total".format(metric),
                           metric_aggregate[metric],
                           dimensions=dims_operations)
                # Save this metric to the cache
                metric_cache[inst_name][rate_name] = {
                    'timestamp': sample_time,
                    'value': metric_aggregate[metric]}

    def _inspect_disk_info(self, insp, inst, inst_name, instance_cache, metric_cache,
                           dims_customer, dims_operations):
        """Inspect disk metrics for an instance"""

        metric_aggregate = {}
        for disk in insp.inspect_disk_info(inst):
            disk_dimensions = {'device': disk[0].device}
            for metric in disk[1]._fields:
                metric_name = "disk.{0}".format(metric)
                value = int(disk[1].__getattribute__(metric))
                metric_aggregate[metric_name] = metric_aggregate.get(
                    metric_name, 0) + value
                this_dimensions = disk_dimensions.copy()
                this_dimensions.update(dims_customer)
                self.gauge(metric_name, value, dimensions=this_dimensions,
                           delegated_tenant=instance_cache.get(inst_name)['tenant_id'],
                           hostname=instance_cache.get(inst_name)['hostname'])
                # Operations (metric name prefixed with "vm."
                this_dimensions = disk_dimensions.copy()
                this_dimensions.update(dims_operations)
                self.gauge("vm.{0}".format(metric_name), value,
                           dimensions=this_dimensions)

        for metric in metric_aggregate:
            self.gauge("{0}_total".format(metric), metric_aggregate[metric],
                       dimensions=dims_customer,
                       delegated_tenant=instance_cache.get(inst_name)['tenant_id'],
                       hostname=instance_cache.get(inst_name)['hostname'])
            self.gauge("vm.{0}_total".format(metric),
                       metric_aggregate[metric],
                       dimensions=dims_operations)

    def _inspect_state(self, insp, inst, inst_name, instance_cache, dims_customer, dims_operations):
        """Look at the state of the instance, publish a metric using a
           user-friendly description in the 'detail' metadata, and return
           a status code (calibrated to UNIX status codes where 0 is OK)
           so that remaining metrics can be skipped if the VM is not OK
        """
        inst_state = inst.state()
        dom_status = inst_state[0] - 1
        metatag = None

        if inst_state[0] in DOM_STATES:
            metatag = {'detail': DOM_STATES[inst_state[0]]}
        # A VM being in SHUTOFF state may have many reasons, we try to be more specific here
        if inst_state[0] == libvirt.VIR_DOMAIN_SHUTOFF:
            if inst_state[1] in DOM_SHUTOFF_STATES:
                metatag = {'detail': DOM_SHUTOFF_STATES[inst_state[1]]}

        self.gauge('host_alive_status', dom_status, dimensions=dims_customer,
                   delegated_tenant=instance_cache.get(inst_name)['tenant_id'],
                   hostname=instance_cache.get(inst_name)['hostname'],
                   value_meta=metatag)
        self.gauge('vm.host_alive_status', dom_status,
                   dimensions=dims_operations,
                   value_meta=metatag)

        return dom_status

    def prepare_run(self):
        """Check if it is time for measurements to be collected"""
        for name, collection in self._collect_intervals.iteritems():
            if collection['period'] <= 0:
                continue

            time_since_last = datetime.now() - collection['last_collect']
            # Handle times that are really close to the collection period
            period_with_fudge_factor = timedelta(0, collection['period'] - 1,
                                                 500000)

            if time_since_last < period_with_fudge_factor:
                self.log.debug('Skipping {} collection for {} seconds'.format(
                               name,
                               (collection['period'] - time_since_last.seconds)))
                collection['skip'] = True
            else:
                collection['skip'] = False
                collection['last_collect'] = datetime.now()

    def _run_ping(self, dims_customer, dims_operations, inst_name, instance_cache, net):
        """Create a ping command and hand it off to the Thread Pool"""
        ping_cmd = self.init_config.get('ping_check').replace('NAMESPACE',
                                                              net['namespace']).split()
        ping_cmd.append(net['ip'])
        dims_customer_ip = dims_customer.copy()
        dims_operations_ip = dims_operations.copy()
        dims_customer_ip['ip'] = net['ip']
        dims_operations_ip['ip'] = net['ip']
        with open(os.devnull, "w") as fnull:
            try:
                self.log.debug("Running ping test: {0}".format(' '.join(ping_cmd)))
                res = subprocess.call(ping_cmd,
                                      stdout=fnull,
                                      stderr=fnull)
                tenant_id = instance_cache.get(inst_name)['tenant_id']
                hostname = instance_cache.get(inst_name)['hostname']
                return (res, dims_customer_ip, dims_operations_ip, tenant_id,
                        hostname)

            except Exception as e:
                self.log.exception("OS error running '{0}' failed".format(ping_cmd), e)
                raise e

    def _check_ping_results(self, ping_results):
        """Iterate through ping results and create measurements"""
        for result in ping_results:
            result.wait()
            # If it wasn't successful, a message was already logged in _run_ping
            if result.successful():
                (res, dims_customer_ip, dims_operations_ip, delegated_tenant,
                 hostname) = result.get()
                self.gauge('ping_status', res, dimensions=dims_customer_ip,
                           delegated_tenant=delegated_tenant,
                           hostname=hostname)
                self.gauge('vm.ping_status', res, dimensions=dims_operations_ip)

    def check(self, instance):
        """Gather VM metrics for each instance"""

        time_start = time.time()

        # Load metric cache
        metric_cache = self._load_metric_cache()

        # Load the nova-obtained instance data cache
        instance_cache = self._load_instance_cache()

        # Build dimensions for both the customer and for operations
        dims_base = self._set_dimensions({'service': 'compute', 'component': 'vm'}, instance)

        # Define aggregate gauges, gauge name to metric name
        agg_gauges = {'vcpus': 'nova.vm.cpu.total_allocated',
                      'ram': 'nova.vm.mem.total_allocated_mb',
                      'disk': 'nova.vm.disk.total_allocated_gb'}
        agg_values = {}
        for gauge in agg_gauges.keys():
            agg_values[gauge] = 0

        insp = inspector.get_hypervisor_inspector()
        updated_cache_this_time = False
        ping_results = []
        for inst in insp._get_connection().listAllDomains():
            # Verify that this instance exists in the cache.  Add if necessary.
            inst_name = inst.name()
            if inst_name not in instance_cache and not updated_cache_this_time:
                #
                # If we have multiple ghost VMs, we'll needlessly
                # update the instance cache.  Let's limit the cache
                # update to once per agent wakeup.
                #
                updated_cache_this_time = True
                instance_cache = self._update_instance_cache()

            # Build customer dimensions
            try:
                dims_customer = dims_base.copy()
                dims_customer['resource_id'] = instance_cache.get(inst_name)['instance_uuid']
                dims_customer['zone'] = instance_cache.get(inst_name)['zone']
                # Add dimensions that would be helpful for operations
                dims_operations = dims_customer.copy()
                dims_operations['tenant_id'] = instance_cache.get(inst_name)['tenant_id']
                dims_operations = self._update_dims_with_metadata(instance_cache, inst_name, dims_operations)
                if self.init_config.get('customer_metadata'):
                    for metadata in self.init_config.get('customer_metadata'):
                        metadata_value = (instance_cache.get(inst_name).
                                          get(metadata))
                        if metadata_value:
                            dims_customer[metadata] = metadata_value
                # Remove customer 'hostname' dimension, this will be replaced by the VM name
                del(dims_customer['hostname'])
            except TypeError:
                # Nova can potentially get into a state where it can't see an
                # instance, but libvirt can.  This would cause TypeErrors as
                # incomplete data is cached for this instance.  Log and skip.
                self.log.error("{0} is not known to nova after instance cache update -- skipping this ghost VM.".format(inst_name))
                continue

            # Accumulate aggregate data
            for gauge in agg_gauges:
                if gauge in instance_cache.get(inst_name):
                    agg_values[gauge] += instance_cache.get(inst_name)[gauge]

            # Skip instances created within the probation period
            vm_probation_remaining = self._test_vm_probation(instance_cache.get(inst_name)['created'])
            if (vm_probation_remaining >= 0):
                self.log.info("Libvirt: {0} in probation for another {1} seconds".format(instance_cache.get(inst_name)['hostname'].encode('utf8'),
                                                                                         vm_probation_remaining))
                continue

            # Skip further processing on VMs that are not in an active state
            if self._inspect_state(insp, inst, inst_name, instance_cache,
                                   dims_customer, dims_operations) != 0:
                continue

            # Skip the remainder of the checks if alive_only is True in the config
            if self.init_config.get('alive_only'):
                continue

            if inst_name not in metric_cache:
                metric_cache[inst_name] = {}

            if self.init_config.get('vm_cpu_check_enable'):
                self._inspect_cpu(insp, inst, inst_name, instance_cache, metric_cache, dims_customer, dims_operations)
            if not self._collect_intervals['disk']['skip']:
                if self.init_config.get('vm_disks_check_enable'):
                    self._inspect_disks(insp, inst, inst_name, instance_cache, metric_cache, dims_customer,
                                        dims_operations)
                if self.init_config.get('vm_extended_disks_check_enable'):
                    self._inspect_disk_info(insp, inst, inst_name, instance_cache, metric_cache, dims_customer,
                                            dims_operations)

            if not self._collect_intervals['vnic']['skip']:
                if self.init_config.get('vm_network_check_enable'):
                    self._inspect_network(insp, inst, inst_name, instance_cache, metric_cache, dims_customer, dims_operations)

            # Memory utilizaion
            # (req. balloon driver; Linux kernel param CONFIG_VIRTIO_BALLOON)
            try:
                mem_stats = inst.memoryStats()
                mem_metrics = {'mem.free_mb': float(mem_stats['unused']) / 1024,
                               'mem.swap_used_mb': float(mem_stats['swap_out']) / 1024,
                               'mem.total_mb': float(mem_stats['available']) / 1024,
                               'mem.used_mb': float(mem_stats['available'] - mem_stats['unused']) / 1024,
                               'mem.free_perc': float(mem_stats['unused']) / float(mem_stats['available']) * 100}
                for name in mem_metrics:
                    self.gauge(name, mem_metrics[name], dimensions=dims_customer,
                               delegated_tenant=instance_cache.get(inst_name)['tenant_id'],
                               hostname=instance_cache.get(inst_name)['hostname'])
                    self.gauge("vm.{0}".format(name), mem_metrics[name],
                               dimensions=dims_operations)
                memory_info = insp.inspect_memory_resident(inst)
                self.gauge('vm.mem.resident_mb', float(memory_info.resident), dimensions=dims_operations)
            except KeyError:
                self.log.debug("Balloon driver not active/available on guest {0} ({1})".format(inst_name,
                                                                                               instance_cache.get(inst_name)['hostname']))
            # Test instance's remote responsiveness (ping check) if possible
            if (self.init_config.get('vm_ping_check_enable')) and self.init_config.get('ping_check') and 'network' in instance_cache.get(inst_name):
                for net in instance_cache.get(inst_name)['network']:
                    ping_args = [dims_customer, dims_operations, inst_name, instance_cache, net]
                    ping_results.append(self.pool.apply_async(self._run_ping, ping_args))

        # Save these metrics for the next collector invocation
        self._update_metric_cache(metric_cache, math.ceil(time.time() - time_start))

        # Publish aggregate metrics
        for gauge in agg_gauges:
            self.gauge(agg_gauges[gauge], agg_values[gauge], dimensions=dims_base)

        # Check results of ping tests
        self._check_ping_results(ping_results)

    def _calculate_rate(self, current_value, cache_value, time_diff):
        """Calculate rate based on current, cache value and time_diff."""
        try:
            rate_value = (current_value - cache_value) / time_diff
        except ZeroDivisionError as e:
            self.log.error("Time difference between current time and "
                           "last_update time is 0 . {0}".format(e))
            #
            # Being extra safe here, in case we divide by zero
            # just skip this reading with check below.
            #
            rate_value = -1
        return rate_value

    def _update_dims_with_metadata(self, instance_cache, inst_name, dim_operations):
        """Update operations dimensions with metadata."""
        dims = dim_operations
        if self.init_config.get('metadata'):
            for metadata in self.init_config.get('metadata'):
                if 'vm_name' == metadata:
                    metadata_value = (instance_cache.get(inst_name).
                                      get('hostname'))
                else:
                    metadata_value = (instance_cache.get(inst_name).
                                      get(metadata))
                if metadata_value:
                    dims[metadata] = metadata_value
        return dims



# (C) Copyright 2015 Hewlett Packard Enterprise Development Company LP
# stdlib
import logging
import psutil
import re

# project
import monasca_agent.collector.checks as checks

log = logging.getLogger(__name__)


class Network(checks.AgentCheck):

    def __init__(self, name, init_config, agent_config):
        super(Network, self).__init__(name, init_config, agent_config)

    def check(self, instance):
        """Capture network metrics

        """
        dimensions = self._set_dimensions(None, instance)

        excluded_ifaces = instance.get('excluded_interfaces', [])
        if excluded_ifaces:
            log.debug('Excluding network devices: {0}'.format(excluded_ifaces))

        exclude_re = instance.get('excluded_interface_re', None)
        if exclude_re:
            exclude_iface_re = re.compile(exclude_re)
            log.debug('Excluding network devices matching: {0}'.format(exclude_re))
        else:
            exclude_iface_re = None

        nics = psutil.net_io_counters(pernic=True)
        for nic_name in nics.keys():
            if self._is_nic_monitored(nic_name, excluded_ifaces, exclude_iface_re):
                nic = nics[nic_name]
                if instance.get('use_bits'):
                    self.rate('net.in_bits_sec', nic.bytes_recv * 8, device_name=nic_name, dimensions=dimensions)
                    self.rate('net.out_bits_sec', nic.bytes_sent * 8, device_name=nic_name, dimensions=dimensions)
                else:
                    self.rate('net.in_bytes_sec', nic.bytes_recv, device_name=nic_name, dimensions=dimensions)
                    self.rate('net.out_bytes_sec', nic.bytes_sent, device_name=nic_name, dimensions=dimensions)
                if instance.get('net_bytes_only'):
                    continue
                self.rate('net.in_packets_sec', nic.packets_recv, device_name=nic_name, dimensions=dimensions)
                self.rate('net.out_packets_sec', nic.packets_sent, device_name=nic_name, dimensions=dimensions)
                self.rate('net.in_errors_sec', nic.errin, device_name=nic_name, dimensions=dimensions)
                self.rate('net.out_errors_sec', nic.errout, device_name=nic_name, dimensions=dimensions)
                self.rate('net.in_packets_dropped_sec', nic.dropin, device_name=nic_name, dimensions=dimensions)
                self.rate('net.out_packets_dropped_sec', nic.dropout, device_name=nic_name, dimensions=dimensions)

                log.debug('Collected 8 network metrics for device {0}'.format(nic_name))

    def _is_nic_monitored(self, nic_name, excluded_ifaces, exclude_iface_re):
        if nic_name in excluded_ifaces:
            return False
        if exclude_iface_re and exclude_iface_re.match(nic_name):
            return False
        return True



import logging
import monasca_setup.detection

log = logging.getLogger(__name__)


class Trove(monasca_setup.detection.ServicePlugin):

    """Detect Trove daemons and setup configuration to monitor them."""

    def __init__(self, template_dir, overwrite=True, args=None):

        service_params = {
            'args': args,
            'template_dir': template_dir,
            'overwrite': overwrite,
            'service_name': 'database',
            'process_names': ['trove-api', 'trove-taskmanager', 'trove-conductor'],
            'service_api_url': "http://localhost:8779",
            'search_pattern': '.*v1.*'
        }

        super(Trove, self).__init__(service_params)



# (C) Copyright 2015 Hewlett Packard Enterprise Development Company LP

"""Detection plugin for the existence of check_mk_agent, whose <<<local>>>
   check output may parsed for metrics."""

import logging
import monasca_setup.agent_config
import os

log = logging.getLogger(__name__)

# List common check_mk_agent locations
check_mk_agent_paths = ['/usr/bin/check_mk_agent',
                        '/usr/local/bin/check_mk_agent']


class CheckMKLocal(monasca_setup.detection.Plugin):
    """Identify existance of check_mk_agent by looking for the executable.
       The check_mk_agent process itself does not need to be running in order
       for this plugin to work, the agent program itself can return local data.
    """

    def _detect(self):
        self.agent_path = None
        for path in check_mk_agent_paths:
            if os.path.isfile(path):
                self.agent_path = path
                self.available = True

    def build_config(self):
        log.info("\tEnabling the check_mk_local plugin")
        config = monasca_setup.agent_config.Plugins()
        config['check_mk_local'] = {'init_config': {'mk_agent_path': self.agent_path},
                                    'instances': [{}]}
        return config



import os
import sys
import inspect

import monasca_agent.collector.checks as checks
import monasca_agent.common.config as configuration
from monasca_agent.common.util import Paths


base_config = configuration.Config(os.path.join(os.path.dirname(__file__),
                                                'test-agent.yaml'))


def load_check(name, config):
    checksd_path = Paths().get_checksd_path()
    if checksd_path not in sys.path:
        sys.path.append(checksd_path)

    check_module = __import__(name)
    check_class = None
    classes = inspect.getmembers(check_module, inspect.isclass)
    for name, clsmember in classes:
        if clsmember == checks.AgentCheck:
            continue
        if issubclass(clsmember, checks.AgentCheck):
            check_class = clsmember
            if checks.AgentCheck in clsmember.__bases__:
                continue
            else:
                break
    if check_class is None:
        raise Exception(
            "Unable to import check %s. Missing a class that inherits "
            "AgentCheck" % name)

    init_config = config.get('init_config', None)
    instances = config.get('instances')

    agent_config = base_config.get_config(sections='Main')
    # init the check class
    try:
        return check_class(
            name, init_config=init_config, agent_config=agent_config,
            instances=instances)
    except Exception:
        # Backwards compatitiblity for old checks that don't support the
        # instances argument.
        c = check_class(name, init_config=init_config,
                        agent_config=agent_config)
        c.instances = instances
        return c



#!/bin/env python
"""Monasca Agent plugin for Check_MK agent
   This plugin will query the Check_MK agent, extracting and publishing
   metrics for all configured local checks, except where configured to discard.
   See check_mk_local.yaml.example for configuration.  For more on Check_MK,
   see http://mathias-kettner.com/checkmk_localchecks.html
"""

# (C) Copyright 2015 Hewlett Packard Enterprise Development Company LP
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

from monasca_agent.collector.checks import AgentCheck


class CheckMK(AgentCheck):
    """Inherit AgentCheck class to process Check_MK checks.
    """

    def __init__(self, name, init_config, agent_config, instances=None):
        AgentCheck.__init__(self, name, init_config, agent_config, instances)

    def check(self, instance):
        """Run check_mk_agent and process the '<<<local>>>' results.
        """

        local = False
        # Run check_mk_agent (in a way compatible with Python 2.6)
        for line in os.popen(self.init_config.get('mk_agent_path')).readlines():
            if local:
                metric_name = None
                dimensions = self._set_dimensions(None, instance)

                # <<<local>>> lines use the following format, space-delimited
                # for the first three fields, then free-form for the fourth:
                # [Status] [Item Name] [Performance data] [Check output]
                check_data = line.split(' ')

                # Look for any custom configuration for this check
                if 'custom' in self.init_config and self.init_config['custom']:
                    custom = filter(lambda d: d['mk_item'] == check_data[1],
                                    self.init_config.get('custom'))
                    if len(custom) > 0:
                        if 'discard' in custom[0] and custom[0]['discard']:
                            continue

                        if 'dimensions' in custom[0]:
                            dimensions = self._set_dimensions(custom[0]['dimensions'],
                                                              instance)
                        if 'metric_name_base' in custom[0]:
                            metric_name = custom[0]['metric_name_base']

                value_meta = {'detail': ' '.join(check_data[3:])}

                # Build a reasonable metric name, if not already configured
                if metric_name is None:
                    metric_name = "check_mk.{0}".format(check_data[1].lower())

                # Send 'status' metric
                self.gauge("{0}.status".format(metric_name), check_data[0],
                           dimensions, value_meta=value_meta)

                # Send performance measurements as separate metrics.  Multiple
                # measurements are separated by a pipe (|) character, and the
                # the value of an individual measurement may contain a list
                # of numbers delmited by semicolons: value;warn;crit;min;max.
                # We will only use the first field, the 'value' field.
                for perf in check_data[2].split('|'):
                    try:
                        measurement = perf.split('=')
                        if len(measurement[1]):
                            self.gauge("{0}.{1}".format(metric_name,
                                                        measurement[0]),
                                       measurement[1].split(';')[0],
                                       dimensions)
                    except IndexError:
                        # Performance field is not well-formed, discard it
                        pass
            else:
                # The '<<<local>>>' section is at the end of the output.  Once
                # reaching this point, process all lines for metrics.
                if line.startswith('<<<local>>>'):
                    local = True



# (C) Copyright 2015 Hewlett Packard Enterprise Development Company LP
# stdlib

import logging
import socket
import urllib2
import urlparse

# project
import monasca_agent.collector.checks as checks
import monasca_agent.collector.checks.services_checks as services_checks
import monasca_agent.collector.checks.utils as utils
import monasca_agent.common.util as util

log = logging.getLogger(__name__)


class Apache(checks.AgentCheck):
    """Tracks basic connection/requests/workers metrics

    See http://httpd.apache.org/docs/2.2/mod/mod_status.html for more details
    """

    GAUGES = {'IdleWorkers': 'apache.performance.idle_worker_count',
              'BusyWorkers': 'apache.performance.busy_worker_count',
              'CPULoad': 'apache.performance.cpu_load_perc',
              'Total kBytes': 'apache.net.total_kbytes',
              'Total Accesses': 'apache.net.hits',
              }
    RATES = {'Total kBytes': 'apache.net.kbytes_sec',
             'Total Accesses': 'apache.net.requests_sec'
             }

    def __init__(self, name, init_config, agent_config, instances=None):
        super(Apache, self).__init__(name, init_config, agent_config, instances)
        self.url = None

    def check(self, instance):
        self.url = instance.get('apache_status_url', None)
        if not self.url:
            raise Exception("Missing 'apache_status_url' in Apache config")

        req = urllib2.Request(self.url, None, util.headers(self.agent_config))
        apache_user = instance.get('apache_user', None)
        apache_password = instance.get('apache_password', None)
        if apache_user and apache_password:
            utils.add_basic_auth(req, apache_user, apache_password)
        else:
            log.debug("Not using authentication for Apache Web Server")

        # Submit a service check for status page availability.
        parsed_url = urlparse.urlparse(self.url)
        apache_host = parsed_url.hostname
        apache_port = str(parsed_url.port or 80)
        service_check_name = 'apache.status'

        # Add additional dimensions
        if apache_host == 'localhost':
            # Localhost is not very useful, so get the actual hostname
            apache_host = socket.gethostname()

        dimensions = self._set_dimensions({'apache_host': apache_host,
                                           'apache_port': apache_port,
                                           'service': 'apache',
                                           'component': 'apache'},
                                          instance)

        try:
            request = urllib2.urlopen(req)
        except Exception as e:
            self.log.info(
                "%s is DOWN, error: %s. Connection failed." % (service_check_name, str(e)))
            self.gauge(service_check_name, 1, dimensions=dimensions)
            return services_checks.Status.DOWN, "%s is DOWN, error: %s. Connection failed." % (
                service_check_name, str(e))
        else:
            self.log.debug("%s is UP" % service_check_name)
            self.gauge(service_check_name, 0, dimensions=dimensions)

        response = request.read()
        metric_count = 0

        # Loop through and extract the numerical values
        for line in response.split('\n'):
            values = line.split(': ')
            if len(values) == 2:  # match
                metric, value = values
                try:
                    value = float(value)
                except ValueError:
                    continue

                # Send metric as a gauge, if applicable
                if metric in self.GAUGES:
                    metric_count += 1
                    metric_name = self.GAUGES[metric]
                    log.debug('Collecting gauge data for: {0}'.format(metric_name))
                    self.gauge(metric_name, value, dimensions=dimensions)

                # Send metric as a rate, if applicable
                if metric in self.RATES:
                    metric_count += 1
                    metric_name = self.RATES[metric]
                    log.debug('Collecting rate data for: {0}'.format(metric_name))
                    self.rate(metric_name, value, dimensions=dimensions)

        if metric_count == 0:
            if self.url[-5:] != '?auto':
                self.url = '%s?auto' % self.url
                self.log.warn("Assuming url was not correct. Trying to add ?auto suffix to the url")
                self.check(instance)
            else:
                return services_checks.Status.DOWN, "%s is DOWN, error: No metrics available.".format(service_check_name)
        else:
            log.debug("Collected {0} metrics for {1} Apache Web Server".format(apache_host, metric_count))



# (c) Copyright 2016 Hewlett Packard Enterprise Development LP

from monasca_setup import agent_config
import monasca_setup.detection
import os


VAR_CACHE_DIR = '/var/cache/monasca_json_plugin'


class JsonPlugin(monasca_setup.detection.ArgsPlugin):
    """Detect if /var/cache/monasca_json_plugin exists

    This builds a config for the json_plugin. This detects if
    /var/cache/monasca_json_plugin exists and if so,
    builds a configuration for it.

    Users are free to add their own configs.
    """
    def __init__(self, template_dir, overwrite=True, args=None):
        super(JsonPlugin, self).__init__(
            template_dir, overwrite, args)

    def _detect(self):
        self.available = False
        if os.path.isdir(VAR_CACHE_DIR):
            self.available = True

    def build_config(self):
        """Build the config as a Plugins object and return."""
        config = agent_config.Plugins()
        config['json_plugin'] = {'init_config': None,
                                 'instances': [{'name': VAR_CACHE_DIR,
                                                'metrics_dir': VAR_CACHE_DIR}]}

        return config

    def dependencies_installed(self):
        """Return True if dependencies are installed."""
        return True



# (C) Copyright 2016 Hewlett Packard Enterprise Development LP

import monasca_agent.collector.checks as checks
from monasca_agent.common.util import timeout_command

LICENSE_STATUS_QUERY = "SELECT COALESCE(" \
                       "(SELECT usage_percent * 100 " \
                       "FROM v_catalog.license_audits " \
                       "WHERE audited_data ='Total' " \
                       "ORDER BY audit_start_timestamp " \
                       "DESC LIMIT 1), 0) license_usage_percent;"

NODE_METRICS_QUERY = "SELECT node_state " \
                     "FROM NODES " \
                     "WHERE node_name = '{0}';"

PROJECTION_METRICS_QUERY = "SELECT projection_name, wos_used_bytes, ros_count, " \
                           "COALESCE(tuple_mover_moveouts, 0) tuple_mover_moveouts, " \
                           "COALESCE(tuple_mover_mergeouts, 0) tuple_mover_mergeouts " \
                           "FROM projection_storage " \
                           "LEFT JOIN (SELECT projection_id, " \
                           "SUM(case when operation_name = 'Moveout' then 1 else 0 end) tuple_mover_moveouts, " \
                           "SUM(case when operation_name = 'Mergeout' then 1 else 0 end) tuple_mover_mergeouts " \
                           "FROM tuple_mover_operations " \
                           "WHERE node_name = '{0}' and is_executing = 't' " \
                           "GROUP BY projection_id) tm " \
                           "ON projection_storage.projection_id = tm.projection_id " \
                           "WHERE node_name = '{0}';"

RESOURCE_METRICS_QUERY = "SELECT COALESCE(request_queue_depth, 0) request_queue_depth, " \
                         "wos_used_bytes, " \
                         "COALESCE(resource_request_reject_count, 0) resource_rejections, " \
                         "COALESCE(disk_space_request_reject_count, 0) disk_space_rejections " \
                         "FROM resource_usage " \
                         "WHERE node_name = '{0}';"

RESOURCE_POOL_METRICS_QUERY = "SELECT pool_name, memory_size_actual_kb, memory_inuse_kb, running_query_count, " \
                              "COALESCE(rejection_count, 0) rejection_count " \
                              "FROM resource_pool_status " \
                              "LEFT JOIN (" \
                              "SELECT pool_id, COUNT(*) rejection_count " \
                              "FROM resource_rejections " \
                              "WHERE node_name = '{0}' " \
                              "GROUP BY pool_id) rj " \
                              "ON resource_pool_status.POOL_OID = rj.POOL_ID " \
                              "WHERE node_name = '{0}'"


class Vertica(checks.AgentCheck):

    def __init__(self, name, init_config, agent_config):
        super(Vertica, self).__init__(name, init_config, agent_config)
        self._last_connection_status = 1

    @staticmethod
    def _get_config(instance):
        user = instance.get('user')
        password = instance.get('password')
        service = instance.get('service')
        node_name = instance.get('node_name')
        timeout = int(instance.get('timeout'))

        return user, password, service, node_name, timeout

    def check(self, instance):
        user, password, service, node_name, timeout = self._get_config(instance)

        dimensions = self._set_dimensions({'component': 'vertica', 'service': service}, instance)
        query = self._build_query(node_name)

        results, connection_status = self._query_database(user, password, timeout, query)

        if connection_status != 0:
            self.gauge('vertica.connection_status', 1, dimensions=dimensions)
            self._last_connection_status = 1
        else:
            if self._last_connection_status > 0:
                # report successful connection status when last status not success
                self.gauge('vertica.connection_status', 0, dimensions=dimensions)
            self._last_connection_status = 0
            results = results.split('\n')

            # Database metrics
            self._report_license_status(results[0], dimensions)

            # Node metrics
            dimensions['node_name'] = node_name

            self._report_node_status(results[1], dimensions)

            self._report_resource_metrics(results[2], dimensions)

            self._report_projection_metrics(results[3], dimensions)

            self._report_resource_pool_metrics(results[4], dimensions)

    def _query_database(self, user, password, timeout, query):
        stdout, stderr, return_code = timeout_command(["/opt/vertica/bin/vsql", "-U", user, "-w", password, "-A", "-R",
                                                       "|", "-t", "-F", ",", "-x"], timeout, command_input=query)
        if return_code == 0:
            # remove trailing newline
            stdout = stdout.rstrip()
            return stdout, 0
        else:
            self.log.error("Error querying vertica with return code of {0} and error {1}".format(return_code, stderr))
            return stderr, 1

    def _build_query(self, node_name):
        query = ''
        query += LICENSE_STATUS_QUERY
        query += NODE_METRICS_QUERY.format(node_name)
        query += RESOURCE_METRICS_QUERY.format(node_name)
        query += PROJECTION_METRICS_QUERY.format(node_name)
        query += RESOURCE_POOL_METRICS_QUERY.format(node_name)
        return query

    def _results_to_dict(self, results):
        return [dict(entry.split(',') for entry in dictionary.split('|')) for dictionary in results.split('||')]

    def _report_node_status(self, results, dimensions):
        result = self._results_to_dict(results)
        node_status = result[0]['node_state']
        status_metric = 0 if node_status == 'UP' else 1
        self.gauge('vertica.node_status', status_metric, dimensions=dimensions, value_meta=result[0])

    def _report_projection_metrics(self, results, dimensions):
        results = self._results_to_dict(results)
        projection_metric_name = 'vertica.projection.'
        for result in results:
            projection_dimensions = dimensions.copy()
            projection_dimensions['projection_name'] = result['projection_name']
            # when nothing has been written, wos_used_bytes is empty.
            # Needs to convert it to zero.
            if not result['wos_used_bytes']:
                result['wos_used_bytes'] = '0'
            self.gauge(projection_metric_name + 'wos_used_bytes', int(result['wos_used_bytes']),
                       dimensions=projection_dimensions)
            self.gauge(projection_metric_name + 'ros_count', int(result['ros_count']), dimensions=projection_dimensions)
            self.rate(projection_metric_name + 'tuple_mover_moveouts', int(result['tuple_mover_moveouts']),
                      dimensions=projection_dimensions)
            self.rate(projection_metric_name + 'tuple_mover_mergeouts', int(result['tuple_mover_mergeouts']),
                      dimensions=projection_dimensions)

    def _report_resource_metrics(self, results, dimensions):
        results = self._results_to_dict(results)
        resource_metric_name = 'vertica.resource.'
        resource_metrics = results[0]
        for metric_name, metric_value in resource_metrics.iteritems():
            if metric_name in ['resource_rejections', 'disk_space_rejections']:
                self.rate(resource_metric_name + metric_name, int(metric_value), dimensions=dimensions)
            else:
                if metric_name == 'wos_used_bytes' and not metric_value:
                    # when nothing has been written, wos_used_bytes is empty.
                    # Needs to convert it to zero.
                    metric_value = '0'
                self.gauge(resource_metric_name + metric_name, int(metric_value), dimensions=dimensions)

    def _report_resource_pool_metrics(self, results, dimensions):
        results = self._results_to_dict(results)
        resource_pool_metric_name = 'vertica.resource.pool.'
        for result in results:
            resource_pool_dimensions = dimensions.copy()
            resource_pool_dimensions['resource_pool'] = result['pool_name']
            self.gauge(resource_pool_metric_name + 'memory_size_actual_kb', int(result['memory_size_actual_kb']),
                       dimensions=resource_pool_dimensions)
            self.gauge(resource_pool_metric_name + 'memory_inuse_kb', int(result['memory_inuse_kb']),
                       dimensions=resource_pool_dimensions)
            self.gauge(resource_pool_metric_name + 'running_query_count', int(result['running_query_count']),
                       dimensions=resource_pool_dimensions)
            self.rate(resource_pool_metric_name + 'rejection_count', int(result['rejection_count']),
                      dimensions=resource_pool_dimensions)

    def _report_license_status(self, results, dimensions):
        results = self._results_to_dict(results)
        license_status = results[0]
        self.gauge('vertica.license_usage_percent', float(license_status['license_usage_percent']),
                   dimensions=dimensions)



import json
import logging
import time

import requests
from requests.packages.urllib3 import exceptions
import six
import warnings

import monasca_agent.collector.checks as checks


LOG = logging.getLogger(__name__)


class SolidFire(checks.AgentCheck):
    """SolidFire plugin for reporting cluster metrics. Reference the general
    plugin documentation for metric specifics.
    """

    def __init__(self, name, init_config, agent_config):
        super(SolidFire, self).__init__(name, init_config, agent_config)
        self.sf = None
        self.instance = None
        self.cluster = None

    def check(self, instance):
        """Pull down cluster stats."""
        self.cluster = instance.get('name')
        dimensions = {'service': 'solidfire',
                      'cluster': self.cluster}
        data = {}
        num_metrics = 0
        # Extract cluster auth information
        auth = self._pull_auth(instance)
        self.sf = SolidFireLib(auth)

        # Query cluster for stats
        data.update(self._get_cluster_stats())
        # Query for active cluster faults.
        data.update(self._list_cluster_faults())
        # Query for cluster capacity info
        data.update(self._get_cluster_capacity())

        # Dump data upstream.
        for key, value in data.iteritems():
            if data[key] is None:
                continue
            self.gauge(key, value, dimensions)
            num_metrics += 1

        LOG.debug('Collected %s metrics' % (num_metrics))

    def _pull_auth(self, instance):
        """Extract auth data from instance data.

        Simple check to verify we have enough auth information to connect
        to the SolidFire cluster.
        """
        for k in ['mvip', 'username', 'password']:
            if k not in instance:
                msg = 'Missing config value: %s' % (k)
                LOG.error(msg)
                raise Exception(msg)
        auth = {'mvip': instance.get('mvip'),
                'port': instance.get('port', 443),
                'login': instance.get('username'),
                'passwd': instance.get('password')}
        auth['url'] = 'https://%s:%s' % (auth['mvip'],
                                         auth['port'])
        return auth

    def _get_cluster_stats(self):
        res = (self.sf.issue_api_request('GetClusterStats', {}, '8.0')
               ['result']['clusterStats'])
        # Cluster utilization is the overall load.
        data = {'solidfire.cluster_utilization': res['clusterUtilization']}
        return data

    def _get_cluster_capacity(self):
        res = (self.sf.issue_api_request('GetClusterCapacity', {}, '8.0')
               ['result']['clusterCapacity'])

        # Number of 4KiB blocks with data after the last garbage collection
        non_zero_blocks = res['nonZeroBlocks']
        # Number of 4KiB blocks without data after the last garbage collection
        zero_blocks = res['zeroBlocks']
        # Number of blocks(not always 4KiB) stored on block drives.
        unique_blocks = res['uniqueBlocks']
        # Amount of space the unique blocks take on the block drives.
        unique_blocks_space = res['uniqueBlocksUsedSpace']

        # Amount of space consumed by the block services, including cruft.
        active_block_space = res['activeBlockSpace']
        # Maximum amount of bytes allocated to the block services.
        max_block_space = res['maxUsedSpace']

        # Amount of space consumed by the metadata services.
        active_slice_space = res['usedMetadataSpace']
        # Amount of space consumed by the metadata services for snapshots.
        active_snap_space = res['usedMetadataSpaceInSnapshots']
        # Maximum amount of bytes allocated to the metadata services.
        max_slice_space = res['maxUsedMetadataSpace']

        # Volume provisioned space
        prov_space = res['provisionedSpace']
        # Max provisionable space if 100% metadata space used.
        max_prov_space = res['maxProvisionedSpace']
        # Overprovision limit.
        max_overprov_space = res['maxOverProvisionableSpace']

        # Number of active iSCSI sessions.
        iscsi_sessions = res['activeSessions']
        # Average IOPS since midnight UTC.
        avg_iops = res['averageIOPS']
        # Peak IOPS since midnight UTC.
        peak_iops = res['peakIOPS']
        # Current IOPs over the last 5 seconds.
        current_iops = res['currentIOPS']
        # Theoretical max IOPS
        max_iops = res['maxIOPS']

        # Single-node clusters can report zero values for some divisors.
        thin_factor, dedup_factor, comp_factor = 1, 1, 1
        # Same calculations used in the SolidFire UI.
        if non_zero_blocks:
            # Thin provisioning factor
            thin_factor = ((non_zero_blocks + zero_blocks) /
                           float(non_zero_blocks))
        if unique_blocks:
            # Data deduplication factor
            dedup_factor = non_zero_blocks / float(unique_blocks)
        if unique_blocks_space:
            # 4096 constant from our internal block size, pre-compression
            # Compression efficiency factor
            comp_factor = (unique_blocks * 4096) / float(unique_blocks_space)
        # Overall data reduction efficiency factor
        eff_factor = thin_factor * dedup_factor * comp_factor

        data = {'solidfire.num_iscsi_sessions': iscsi_sessions,
                'solidfire.iops.avg_utc': avg_iops,
                'solidfire.iops.peak_utc': peak_iops,
                'solidfire.iops.avg_5_sec': current_iops,
                'solidfire.iops.max_available': max_iops,
                'solidfire.provisioned_bytes': prov_space,
                'solidfire.max_provisioned_bytes': max_prov_space,
                'solidfire.max_overprovisioned_bytes': max_overprov_space,
                'solidfire.max_block_bytes': max_block_space,
                'solidfire.active_block_bytes': active_block_space,
                'solidfire.max_meta_bytes': max_slice_space,
                'solidfire.active_meta_bytes': active_slice_space,
                'solidfire.active_snapshot_bytes': active_snap_space,
                'solidfire.non_zero_blocks': non_zero_blocks,
                'solidfire.zero_blocks': zero_blocks,
                'solidfire.unique_blocks': unique_blocks,
                'solidfire.unique_blocks_used_bytes': unique_blocks_space,
                'solidfire.thin_provision_factor': thin_factor,
                'solidfire.deduplication_factor': dedup_factor,
                'solidfire.compression_factor': comp_factor,
                'solidfire.data_reduction_factor': eff_factor
                }
        return data

    def _list_cluster_faults(self):
        # Report the number of active faults. Might be useful for an alarm?
        res = (self.sf.issue_api_request('ListClusterFaults',
                                         {'faultTypes': 'current'},
                                         '8.0')
               ['result']['faults'])
        data = {'solidfire.active_cluster_faults': len(res)}
        return data


def retry(exc_tuple, tries=5, delay=1, backoff=2):
    # Retry decorator used for issuing API requests.
    def retry_dec(f):
        @six.wraps(f)
        def func_retry(*args, **kwargs):
            _tries, _delay = tries, delay
            while _tries > 1:
                try:
                    return f(*args, **kwargs)
                except exc_tuple:
                    time.sleep(_delay)
                    _tries -= 1
                    _delay *= backoff
                    LOG.debug('Retrying %(args)s, %(tries)s attempts '
                              'remaining...',
                              {'args': args, 'tries': _tries})
            msg = ('Retry count exceeded for command: %s' %
                   (args[1]))
            LOG.error(msg)
            raise Exception(msg)
        return func_retry
    return retry_dec


class SolidFireLib(object):
    """Gutted version of the Cinder driver.

    Just enough to communicate with a SolidFire cluster for POC.
    """

    retryable_errors = ['xDBVersionMismatch',
                        'xMaxSnapshotsPerVolumeExceeded',
                        'xMaxClonesPerVolumeExceeded',
                        'xMaxSnapshotsPerNodeExceeded',
                        'xMaxClonesPerNodeExceeded',
                        'xNotReadyForIO']

    retry_exc_tuple = (requests.exceptions.ConnectionError)

    def __init__(self, auth):
        self.endpoint = auth
        self.active_cluster_info = {}
        self._set_active_cluster_info(auth)

    @retry(retry_exc_tuple, tries=6)
    def issue_api_request(self, method, params, version='1.0', endpoint=None):
        if params is None:
            params = {}
        if endpoint is None:
            endpoint = self.active_cluster_info['endpoint']

        payload = {'method': method, 'params': params}
        url = '%s/json-rpc/%s/' % (endpoint['url'], version)
        with warnings.catch_warnings():
            warnings.simplefilter("ignore", exceptions.InsecureRequestWarning)
            req = requests.post(url,
                                data=json.dumps(payload),
                                auth=(endpoint['login'], endpoint['passwd']),
                                verify=False,
                                timeout=30)
        response = req.json()
        req.close()
        if (('error' in response) and
                (response['error']['name'] in self.retryable_errors)):
            msg = ('Retryable error (%s) encountered during '
                   'SolidFire API call.' % response['error']['name'])
            raise Exception(msg)

        if 'error' in response:
            msg = ('API response: %s') % response
            raise Exception(msg)

        return response

    def _set_active_cluster_info(self, endpoint):
        self.active_cluster_info['endpoint'] = endpoint

        for k, v in self.issue_api_request(
                'GetClusterInfo',
                {})['result']['clusterInfo'].items():
            self.active_cluster_info[k] = v

        # Add a couple extra things that are handy for us
        self.active_cluster_info['clusterAPIVersion'] = (
            self.issue_api_request('GetClusterVersionInfo',
                                   {})['result']['clusterAPIVersion'])



import unittest
from tests.common import load_check
import time


class TestMySql(unittest.TestCase):

    def setUp(self):
        # This should run on pre-2.7 python so no skiptest
        self.skip = False
        try:
            import pymysql
        except ImportError:
            self.skip = True

    def testChecks(self):
        if not self.skip:
            agent_config = {'mysql_server': 'localhost',
                            'mysql_user': "datadog",
                            'mysql_pass': "phQOrbaXem0kP8JHri1qSMRS",
                            'version': '0.1',
                            'api_key': 'toto'}

            # Initialize the check from checks_d
            c = load_check('mysql', {'init_config': {}, 'instances': {}}, agent_config)
            conf = c.parse_agent_config(agent_config)
            self.check = load_check('mysql', conf, agent_config)

            self.check.run()
            metrics = self.check.get_metrics()
            self.assertTrue(len(metrics) >= 8, metrics)
            time.sleep(1)
            self.check.run()
            metrics = self.check.get_metrics()
            self.assertTrue(len(metrics) >= 16, metrics)

if __name__ == '__main__':
    unittest.main()



#!/bin/env python
# (C) Copyright 2015,2016 Hewlett Packard Enterprise Development LP
"""Monitoring Agent remote host aliveness checker.

"""

import socket
import subprocess
import sys

import monasca_agent.collector.checks.services_checks as services_checks
import monasca_agent.common.util as util


class HostAlive(services_checks.ServicesCheck):

    """Inherit ServicesCheck class to test if a host is alive or not.

    """

    def __init__(self, name, init_config, agent_config, instances=None):
        super(HostAlive, self).__init__(name, init_config, agent_config, instances)

    def _test_ssh(self, host, port, timeout=None):
        """Connect to the SSH port (typically 22) and look for a banner.

        """
        if port is None:
            port = 22
        try:
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            if timeout is not None:
                sock.settimeout(timeout)
        except socket.error as msg:
            error_message = 'Error creating socket: {0}'.format(str(msg[0]) + msg[1])
            self.log.warn(error_message)
            return False, error_message

        try:
            host_ip = socket.gethostbyname(host)
        except socket.gaierror:
            error_message = 'Unable to resolve host {0}'.format(host)
            self.log.warn(error_message)
            return False, error_message

        try:
            sock.connect((host_ip, port))
            banner = sock.recv(1024)
            sock.close()
        except socket.error:
            error_message = 'Unable to open socket to host {0}'.format(host)
            self.log.warn(error_message)
            return False, error_message
        if banner.startswith('SSH'):
            return True, None
        else:
            error_message = 'Unexpected response "{0}" from host {1}'.format(banner, host)
            self.log.warn(error_message)
            return False, error_message

    def _test_ping(self, host, timeout=None):
        """Attempt to ping the host.

        """
        ping_prefix = "ping -c 1 -q "
        if timeout is not None:
            ping_prefix += "-W " + str(timeout) + " "
        if sys.platform.startswith('win'):
            ping_prefix = "ping -n 1 "
            if timeout is not None:
                # On Windows, timeout is in milliseconds
                timeout *= 1000
                ping_prefix += "-w " + str(timeout) + " "
        ping_command = ping_prefix + host

        try:
            subprocess.check_output(ping_command.split(" "), stderr=subprocess.STDOUT)
        except subprocess.CalledProcessError:
            error_message = 'Host not accessible, ping test failed ("{0}")'.format(ping_command)
            self.log.info(error_message)
            return False, error_message
        except OSError as err:
            error_message = 'ping command "{0}" failed to run: {1}'.format(ping_command, err)
            self.log.warn(error_message)
            return False, error_message
        return True, None

    def _check(self, instance):
        """Run the desired host-alive check against this host.

        """

        host_name = instance.get('host_name', None)
        if not host_name:
            raise ValueError('host_name not specified!')

        # Allow a different network name to be used for the check
        # to handle multi-homed systems
        if instance.get('target_hostname', None):
            target_hostname = instance.get('target_hostname')
        else:
            target_hostname = host_name

        host_dimensions = {'hostname': host_name, 'observer_host': util.get_hostname()}
        # If the check is against a different network name than host_name, add it to
        # the dimensions
        if target_hostname != host_name:
            host_dimensions['target_hostname'] = target_hostname

        dimensions = self._set_dimensions(host_dimensions,
                                          instance)

        success = False
        test_type = instance['alive_test']
        if test_type == 'ssh':
            success, error_message = self._test_ssh(target_hostname,
                                                    self.init_config.get('ssh_port'),
                                                    self.init_config.get('ssh_timeout'))
        elif test_type == 'ping':
            success, error_message = self._test_ping(target_hostname,
                                                     self.init_config.get('ping_timeout'))
        else:
            error_message = 'Unrecognized alive_test: {0}'.format(test_type)

        dimensions.update({'test_type': test_type})
        if success is True:
            self.gauge('host_alive_status',
                       0,
                       dimensions=dimensions)
            return services_checks.Status.UP, "UP"
        else:
            self.gauge('host_alive_status',
                       1,
                       dimensions=dimensions,
                       value_meta={'error': error_message})
            return services_checks.Status.DOWN, "DOWN"



# (C) Copyright 2015-2016 Hewlett Packard Enterprise Development LP

import logging
import urllib2

import monasca_setup.agent_config
import monasca_setup.detection

from monasca_setup.detection.utils import service_api_check

log = logging.getLogger(__name__)

# Defaults
rabbit_conf = '/root/.rabbitmq.cnf'
rabbitmq_api_url = 'http://localhost:15672/api/'


class RabbitMQ(monasca_setup.detection.Plugin):
    """Detect RabbitMQ daemons and setup configuration to monitor them.

        This plugin needs user/pass info for rabbitmq setup. There are two
        ways to provide it, either by a file placed in /root/.rabbit.cnf or
        by specifying the following arguments:
            - api_url
            - user
            - password
            - queues
            - nodes
            - exchanges
        queues, exchanges and nodes are a comma separated list and are optional

        The file at /root/.rabbit.cnf should have this format:
        [client]
            user = guest
            password = guest
            nodes=rabbit@localhost, rabbit2@domain
            exchanges=nova, cinder, neutron
    """
    def __init__(self, *args, **kwargs):
        super(RabbitMQ, self).__init__(*args, **kwargs)
        if self.args is None:
            self._watch_api = False
        else:
            self._watch_api = self.args.pop('watch_api', False)
        if type(self._watch_api) is str:
            self._watch_api = (self._watch_api.lower() == 'true')

    def _detect(self):
        """Run detection, set self.available True if the service is detected.
        """
        if monasca_setup.detection.find_process_service('rabbitmq-server') is not None:
            self.available = True

    def _get_config(self):
        """Set the configuration to be used for connecting to rabbitmq
        :return:
        """
        # Set defaults and read config or use arguments
        if self.args is None or not self.args:
            self.api_url = rabbitmq_api_url
            self.user = 'guest'
            self.password = 'guest'
            self.queues = None
            self.nodes = None
            self.exchanges = None

            self._read_config(rabbit_conf)
        else:
            self.api_url = self.args.get('api_url', rabbitmq_api_url)
            self.user = self.args.get('user', 'guest')
            self.password = self.args.get('password', 'guest')
            self.queues = self.args.get('queues')
            self.nodes = self.args.get('nodes')
            self.exchanges = self.args.get('exchanges')

    def _login_test(self):
        """Attempt to log into the rabbitmq admin api to verify credentials.
        :return: bool status of the test
        """
        url = self.api_url + '/aliveness-test/%2F'
        password_mgr = urllib2.HTTPPasswordMgrWithDefaultRealm()
        password_mgr.add_password(None,
                                  self.api_url,
                                  self.user,
                                  self.password)
        handler = urllib2.HTTPBasicAuthHandler(password_mgr)
        opener = urllib2.build_opener(handler)

        request = opener.open(url)
        response = request.read()
        request.close()
        if '{"status":"ok"}' in response:
            return True
        else:
            return False

    def _read_config(self, config_file):
        """Read the configuration setting member variables as appropriate.
        :param config_file: The filename of the configuration to read and parse
        """
        # Read the rabbitmq config file to extract the needed variables.
        client_section = False
        with open(config_file, "r") as conf:
            for row in conf:
                if "[client]" in row:
                    client_section = True
                    log.info("\tUsing client credentials from {:s}".format(config_file))
                    pass
                if client_section:
                    if "api_url=" in row:
                        self.api_url = row.split("=")[1].strip()
                    if "user=" in row:
                        self.user = row.split("=")[1].strip()
                    if "password=" in row:
                        self.password = row.split("=")[1].strip()
                    if "exchanges=" in row:
                        self.exchanges = row.split("=")[1].strip()
                    if "queues=" in row:
                        self.queues = row.split("=")[1].strip()
                    if "nodes=" in row:
                        self.nodes = row.split("=")[1].strip()

    @staticmethod
    def _split_list(to_split):
        return [x.strip() for x in to_split.split(',')]

    def build_config(self):
        """Build the config as a Plugins object and return.
        """
        config = monasca_setup.agent_config.Plugins()
        # First watch the process
        config.merge(monasca_setup.detection.watch_process(['epmd'], 'rabbitmq', detailed=False))
        log.info("\tWatching the rabbitmq-server process.")
        config.merge(monasca_setup.detection.watch_process_by_username('rabbitmq', 'rabbitmq', 'rabbitmq'))
        log.info("\tWatching all processes owned by the rabbitmq user.")

        if not self._watch_api:
            return config

        try:
            self._get_config()
            # Then watch the http status check
            service_name = 'rabbitmq'

            # Setup an active http_status check on the API
            if self.api_url.endswith("/"):
                check_api_url = self.api_url
            else:
                check_api_url = self.api_url + "/"

            log.info("\tConfiguring an http_check for the {0} API.".format(
                service_name))
            config.merge(service_api_check(service_name,
                                           check_api_url,
                                           '.*RabbitMQ.*',
                                           use_keystone=False,
                                           service=service_name))
            if self._login_test():
                instance_config = {'name': self.api_url,
                                   'rabbitmq_api_url': self.api_url,
                                   'rabbitmq_user': self.user,
                                   'rabbitmq_pass': self.password}
                if self.queues is not None:
                    instance_config['queues'] = self._split_list(self.queues)
                if self.exchanges is not None:
                    instance_config['exchanges'] = self._split_list(self.exchanges)
                if self.nodes is not None:
                    instance_config['nodes'] = self._split_list(self.nodes)

                config['rabbitmq'] = {'init_config': None, 'instances': [instance_config]}
            else:
                exception_msg = 'Unable to access the RabbitMQ admin URL; ' \
                                'the RabbitMQ plugin is not configured. ' \
                                'Please correct and re-run monasca-setup.'
                log.error(exception_msg)
                raise Exception(exception_msg)
        except Exception:
            exception_msg = 'Error configuring the RabbitMQ check plugin'
            log.exception(exception_msg)
            raise Exception(exception_msg)

        return config

    def dependencies_installed(self):
        return True



# (C) Copyright 2015 Hewlett Packard Enterprise Development Company LP

import monasca_setup.detection


class Keystone(monasca_setup.detection.ServicePlugin):

    """Detect Keystone daemons and setup configuration to monitor them.

    """

    def __init__(self, template_dir, overwrite=True, args=None):
        service_params = {
            'args': args,
            'template_dir': template_dir,
            'overwrite': overwrite,
            'service_name': 'identity-service',
            'process_names': ['keystone-admin',
                              'keystone-main',
                              'keystone-all'],
            'service_api_url': 'http://localhost:35357/v3',
            'search_pattern': '.*v3\..*'
        }

        super(Keystone, self).__init__(service_params)



import unittest
import mock

from monasca_agent.common.keystone import Keystone
from tests.common import base_config


class TestKeystone(unittest.TestCase):

    default_endpoint_type = 'publicURL'
    default_service_type = 'monitoring'

    def testKeyStoneIsSingleton(self):
        keystone_1 = Keystone({})
        keystone_2 = Keystone({})
        keystone_3 = Keystone({})

        self.assertTrue(keystone_1 is keystone_2)
        self.assertTrue(keystone_1 is keystone_3)

    def testServiceCatalogMonascaUrlUsingDefaults(self):
        Keystone.instance = None
        with mock.patch('keystoneclient.v3.client.Client') as client:
            config = base_config.get_config('Api')
            keystone = Keystone(config)
            keystone.get_monasca_url()
            self.assertTrue(client.called)
            self.assertIn('auth_url', client.call_args[client.call_count])
            self.assertNotIn('service_type', client.call_args[client.call_count])
            self.assertNotIn('endpoint_type', client.call_args[client.call_count])
            self.assertNotIn('region_name', client.call_args[client.call_count])
            client.return_value.service_catalog.url_for.assert_has_calls([
                mock.call(endpoint_type=self.default_endpoint_type, service_type=self.default_service_type)
            ])

    def testServiceCatalogMonascaUrlWithCustomServiceType(self):
        Keystone.instance = None
        service_type = 'my_service_type'
        with mock.patch('keystoneclient.v3.client.Client') as client:
            config = base_config.get_config('Api')
            config.update({'service_type': service_type})
            keystone = Keystone(config)
            keystone.get_monasca_url()
            self.assertTrue(client.called)
            self.assertIn('auth_url', client.call_args[client.call_count])
            self.assertNotIn('service_type', client.call_args[client.call_count])
            self.assertNotIn('endpoint_type', client.call_args[client.call_count])
            self.assertNotIn('region_name', client.call_args[client.call_count])
            client.return_value.service_catalog.url_for.assert_has_calls([
                mock.call(endpoint_type=self.default_endpoint_type, service_type=service_type)
            ])

    def testServiceCatalogMonascaUrlWithCustomEndpointType(self):
        Keystone.instance = None
        endpoint_type = 'internalURL'
        with mock.patch('keystoneclient.v3.client.Client') as client:
            config = base_config.get_config('Api')
            config.update({'endpoint_type': endpoint_type})
            keystone = Keystone(config)
            keystone.get_monasca_url()
            self.assertTrue(client.called)
            self.assertIn('auth_url', client.call_args[client.call_count])
            self.assertNotIn('service_type', client.call_args[client.call_count])
            self.assertNotIn('endpoint_type', client.call_args[client.call_count])
            self.assertNotIn('region_name', client.call_args[client.call_count])
            client.return_value.service_catalog.url_for.assert_has_calls([
                mock.call(endpoint_type=endpoint_type, service_type=self.default_service_type)
            ])

    def testServiceCatalogMonascaUrlWithCustomRegionName(self):
        Keystone.instance = None
        region_name = 'my_region'
        with mock.patch('keystoneclient.v3.client.Client') as client:
            config = base_config.get_config('Api')
            config.update({'region_name': region_name})
            keystone = Keystone(config)
            keystone.get_monasca_url()
            self.assertTrue(client.called)
            self.assertIn('auth_url', client.call_args[client.call_count])
            self.assertNotIn('service_type', client.call_args[client.call_count])
            self.assertNotIn('endpoint_type', client.call_args[client.call_count])
            self.assertNotIn('region_name', client.call_args[client.call_count])
            client.return_value.service_catalog.url_for.assert_has_calls([
                mock.call(endpoint_type=self.default_endpoint_type, service_type=self.default_service_type,
                          attr='region', filter_value=region_name)
            ])

    def testServiceCatalogMonascaUrlWithThreeCustomParameters(self):
        Keystone.instance = None
        endpoint_type = 'internalURL'
        service_type = 'my_service_type'
        region_name = 'my_region'
        with mock.patch('keystoneclient.v3.client.Client') as client:
            config = base_config.get_config('Api')
            config.update({'endpoint_type': endpoint_type})
            config.update({'service_type': service_type})
            config.update({'region_name': region_name})
            keystone = Keystone(config)
            keystone.get_monasca_url()
            self.assertTrue(client.called)
            self.assertIn('auth_url', client.call_args[client.call_count])
            self.assertNotIn('service_type', client.call_args[client.call_count])
            self.assertNotIn('endpoint_type', client.call_args[client.call_count])
            self.assertNotIn('region_name', client.call_args[client.call_count])
            client.return_value.service_catalog.url_for.assert_has_calls([
                mock.call(endpoint_type=endpoint_type, service_type=service_type,
                          attr='region', filter_value=region_name)
            ])



#!/bin/env python
# (C) Copyright 2015-2016 Hewlett Packard Enterprise Development LP
"""Monitoring Agent plugin for HTTP/API checks.

"""

import os
import re
import socket
import sys
import time

from httplib2 import Http
from httplib2 import httplib
from httplib2 import HttpLib2Error

import monasca_agent.collector.checks.services_checks as services_checks
import monasca_agent.common.config as cfg
import monasca_agent.common.keystone as keystone


class HTTPCheck(services_checks.ServicesCheck):

    def __init__(self, name, init_config, agent_config, instances=None):
        self._socket_errors = set()
        self._response_not_ready = set()
        self._general_exception = set()
        self._invalid_token = set()
        self._warn_msg = set()

        super(HTTPCheck, self).__init__(name, init_config, agent_config,
                                        instances)
        # init the keystone client if instance has use_keystone
        self._api_config = cfg.Config().get_config('Api')
        self._ksclients = {}
        for instance in instances:
            addr, username, password, timeout, headers, response_time, \
                disable_ssl_validation, use_keystone, keystone_config, \
                instance_name = self._load_http_conf(instance)
            if use_keystone:
                # keystone is a singleton. It will be initialized once,
                # the first config instance used.
                if keystone_config:
                    ksclient = keystone.Keystone(keystone_config)
                else:
                    ksclient = keystone.Keystone(self._api_config)
                self._ksclients[instance_name] = ksclient

    @staticmethod
    def _load_http_conf(instance):
        # Fetches the conf
        instance_name = instance.get('name', None)
        if instance_name is None:
            raise Exception("Bad configuration. You must specify a name")
        username = instance.get('username', None)
        password = instance.get('password', None)
        timeout = int(instance.get('timeout', 10))
        headers = instance.get('headers', {})
        headers.setdefault('User-Agent', os.path.basename(sys.argv[0]))
        use_keystone = instance.get('use_keystone', False)
        keystone_config = instance.get('keystone_config', None)
        url = instance.get('url', None)
        response_time = instance.get('collect_response_time', False)
        if url is None:
            raise Exception("Bad configuration. You must specify a url")
        ssl = instance.get('disable_ssl_validation', True)

        return url, username, password, timeout, headers, response_time, \
            ssl, use_keystone, keystone_config, instance_name

    def _http_check(self, instance):
        addr, username, password, timeout, headers, response_time, \
            disable_ssl_validation, use_keystone, keystone_config, \
            instance_name = self._load_http_conf(instance)
        dimensions = self._set_dimensions({'url': addr}, instance)

        start = time.time()

        done = False
        retry = False
        while not done or retry:
            if use_keystone:
                ksclient = self._ksclients[instance_name]
                token = ksclient.get_token()
                if token:
                    headers["X-Auth-Token"] = token
                    headers["Content-type"] = "application/json"
                else:
                    error_msg = "Unable to get token. Keystone API server may be down."
                    warn_string = '{0} Skipping check for {1}'.format(error_msg, addr)
                    self.log.warning(warn_string)
                    return False, error_msg
            try:
                self.log.debug("Connecting to %s" % addr)
                if disable_ssl_validation:
                    self.log.info("Skipping SSL certificate validation for %s based on configuration" % addr)
                h = Http(timeout=timeout, disable_ssl_certificate_validation=disable_ssl_validation)
                if username is not None and password is not None:
                    h.add_credentials(username, password)
                resp, content = h.request(addr, "GET", headers=headers)

            except (socket.timeout, HttpLib2Error, socket.error) as e:
                length = int((time.time() - start) * 1000)
                error_msg = 'error: {0}. Connection failed after {1} ' \
                            'ms'.format(repr(e), length)
                if addr not in self._socket_errors:
                    self._socket_errors.add(addr)
                    warn_string = '{0} is DOWN, {1}'.format(addr, error_msg)
                    self.log.warn(warn_string)
                return False, error_msg

            except httplib.ResponseNotReady as e:
                length = int((time.time() - start) * 1000)
                error_msg = 'error: {0}. Network is not routable after {1} ' \
                            'ms'.format(repr(e), length)
                if addr not in self._response_not_ready:
                    self._response_not_ready.add(addr)
                    warn_string = '{0} is DOWN, {1}'.format(addr, error_msg)
                    self.log.warn(warn_string)
                return False, error_msg

            except Exception as e:
                length = int((time.time() - start) * 1000)
                error_msg = 'error: {0}. Connection failed after {1} ms'.format(repr(e), length)
                if addr not in self._general_exception:
                    self._general_exception.add(addr)
                    error_string = '{0} is DOWN, {1}'.format(addr, error_msg)
                    self.log.error(error_string)
                return False, error_msg

            if response_time:
                # Stop the timer as early as possible
                running_time = time.time() - start
                self.gauge('http_response_time', running_time, dimensions=dimensions)

            if int(resp.status) >= 400:
                if use_keystone and int(resp.status) == 401:
                    if retry:
                        error_msg = 'unable to get a valid token to connect with'
                        if addr not in self._invalid_token:
                            self._invalid_token.add(addr)
                            error_string = '{0} is DOWN, {1}'.format(addr, error_msg)
                            self.log.error(error_string)
                        return False, error_msg
                    else:
                        # Get a new token and retry
                        self.log.info("Token expired, getting new token and retrying...")
                        retry = True
                        ksclient.refresh_token()
                        continue
                else:
                    warn_msg = 'error code: {0}'.format(str(resp.status))
                    if addr not in self._warn_msg:
                        self._warn_msg.add(addr)
                        warn_string = '{0} is DOWN, {1}'.format(addr, warn_msg)
                        self.log.warn(warn_string)
                    return False, warn_msg

            self._socket_errors.discard(addr)
            self._invalid_token.discard(addr)
            self._response_not_ready.discard(addr)
            self._general_exception.discard(addr)
            self._warn_msg.discard(addr)

            done = True
            return True, content

    def _check(self, instance):
        addr = instance.get("url", None)
        pattern = instance.get('match_pattern', None)

        dimensions = self._set_dimensions({'url': addr}, instance)

        success, result_string = self._http_check(instance)
        if not success:
            # maximum length of value_meta including {'error':''} is 2048
            # Cutting it down to 1024 here so we don't clutter the
            # database too much.
            self.gauge('http_status',
                       1,
                       dimensions=dimensions,
                       value_meta={'error': result_string[:1024]})
            return services_checks.Status.DOWN, result_string

        if pattern is not None:
            if re.search(pattern, result_string, re.DOTALL):
                self.log.debug("Pattern match successful")
            else:
                error_string = 'Pattern match failed! "{0}" not in "{1}"'.format(pattern, result_string)
                self.log.info(error_string)
                # maximum length of value_meta including {'error':''} is 2048
                # Cutting it down to 1024 here so we don't clutter the
                # database too much.
                self.gauge('http_status',
                           1,
                           dimensions=dimensions,
                           value_meta={'error': error_string[:1024]})
                return services_checks.Status.DOWN, error_string

        success_string = '{0} is UP'.format(addr)
        self.log.debug(success_string)
        self.gauge('http_status', 0, dimensions=dimensions)
        return services_checks.Status.UP, success_string



# (C) Copyright 2015 Hewlett Packard Enterprise Development Company LP

from monasca_agent.collector.checks import AgentCheck
from monasca_agent.common.exceptions import CheckException


class ShouldRestartException(Exception):
    pass


class PostgreSql(AgentCheck):

    """Collects per-database, and optionally per-relation metrics.

    """

    RATE = AgentCheck.rate
    GAUGE = AgentCheck.gauge

    # turning columns into dimensions
    DB_METRICS = {
        'descriptors': [('datname', 'db')],
        'metrics': {'numbackends': ('postgresql.connections', GAUGE),
                    'xact_commit': ('postgresql.commits', RATE),
                    'xact_rollback': ('postgresql.rollbacks', RATE),
                    'blks_read': ('postgresql.disk_read', RATE),
                    'blks_hit': ('postgresql.buffer_hit', RATE),
                    'tup_returned': ('postgresql.rows_returned', RATE),
                    'tup_fetched': ('postgresql.rows_fetched', RATE),
                    'tup_inserted': ('postgresql.rows_inserted', RATE),
                    'tup_updated': ('postgresql.rows_updated', RATE),
                    'tup_deleted': ('postgresql.rows_deleted', RATE)},
        'query': """
SELECT datname,
       %s
  FROM pg_stat_database
 WHERE datname not ilike 'template%%'
   AND datname not ilike 'postgres'
""",
        'relation': False,
    }

    NEWER_92_METRICS = {
        'deadlocks': ('postgresql.deadlocks', GAUGE),
        'temp_bytes': ('postgresql.temp_bytes', RATE),
        'temp_files': ('postgresql.temp_files', RATE),
    }

    REL_METRICS = {
        'descriptors': [('relname', 'table')],
        'metrics': {'seq_scan': ('postgresql.seq_scans', RATE),
                    'seq_tup_read': ('postgresql.seq_rows_read', RATE),
                    'idx_scan': ('postgresql.index_scans', RATE),
                    'idx_tup_fetch': ('postgresql.index_rows_fetched', RATE),
                    'n_tup_ins': ('postgresql.rows_inserted', RATE),
                    'n_tup_upd': ('postgresql.rows_updated', RATE),
                    'n_tup_del': ('postgresql.rows_deleted', RATE),
                    'n_tup_hot_upd': ('postgresql.rows_hot_updated', RATE),
                    'n_live_tup': ('postgresql.live_rows', GAUGE),
                    'n_dead_tup': ('postgresql.dead_rows', GAUGE)},
        'query': """
SELECT relname,
       %s
  FROM pg_stat_user_tables
 WHERE relname = ANY(%s)""",
        'relation': True,
    }

    IDX_METRICS = {
        'descriptors': [
            ('relname', 'table'),
            ('indexrelname', 'index')
        ],
        'metrics': {
            'idx_scan': ('postgresql.index_scans', RATE),
            'idx_tup_read': ('postgresql.index_rows_read', RATE),
            'idx_tup_fetch': ('postgresql.index_rows_fetched', RATE),
        },
        'query': """
SELECT relname,
       indexrelname,
       %s
  FROM pg_stat_user_indexes
 WHERE relname = ANY(%s)""",
        'relation': True,
    }

    def __init__(self, name, init_config, agent_config):
        AgentCheck.__init__(self, name, init_config, agent_config)
        self.dbs = {}
        self.versions = {}

    @staticmethod
    def get_library_versions():
        try:
            import psycopg2
            version = psycopg2.__version__
        except ImportError:
            version = "Not Found"
        except AttributeError:
            version = "Unknown"
        return {"psycopg2": version}

    def _get_version(self, key, db):
        if key not in self.versions:
            cursor = db.cursor()
            cursor.execute('SHOW SERVER_VERSION;')
            result = cursor.fetchone()
            try:
                version = map(int, result[0].split('.'))
            except Exception:
                version = result[0]
            self.versions[key] = version

        return self.versions[key]

    def _is_9_2_or_above(self, key, db):
        version = self._get_version(key, db)
        if isinstance(version, list):
            return version >= [9, 2, 0]

        return False

    def _collect_stats(self, key, db, dimensions, relations):
        """Query pg_stat_* for various metrics

        If relations is not an empty list, gather per-relation metrics
        on top of that.
        """
        from psycopg2 import InterfaceError

        # Extended 9.2+ metrics
        if self._is_9_2_or_above(key, db):
            self.DB_METRICS['metrics'].update(self.NEWER_92_METRICS)

        # Do we need relation-specific metrics?
        if not relations:
            metric_scope = (self.DB_METRICS,)
        else:
            metric_scope = (self.DB_METRICS, self.REL_METRICS, self.IDX_METRICS)

        for scope in metric_scope:
            # build query
            cols = scope['metrics'].keys()  # list of metrics to query, in some order
            # we must remember that order to parse results
            try:
                cursor = db.cursor()
            except InterfaceError as e:
                self.log.error("Connection seems broken: %s" % str(e))
                raise ShouldRestartException

            # if this is a relation-specific query, we need to list all relations last
            if scope['relation'] and len(relations) > 0:
                query = scope['query'] % (", ".join(cols), "%s")  # Keep the last %s intact
                self.log.debug("Running query: %s with relations: %s" % (query, relations))
                cursor.execute(query, (relations, ))
            else:
                query = scope['query'] % (", ".join(cols))
                self.log.debug("Running query: %s" % query)
                cursor.execute(query)

            results = cursor.fetchall()
            cursor.close()

            # parse & submit results
            # A row should look like this
            # (descriptor, descriptor, ..., value, value, value, value, ...)
            # with descriptor a PG relation or index name, which we use to create the dimensions
            for row in results:
                # turn descriptors into dimensions
                desc = scope['descriptors']
                # Check that all columns will be processed
                assert len(row) == len(cols) + len(desc)

                # Build dimensions
                # descriptors are: (pg_name, dd_tag_name): value
                # Special-case the "db" tag, which overrides the one that is passed as instance_dimensions
                # The reason is that pg_stat_database returns all databases regardless of the
                # connection.
                if not scope['relation'] and 'db' in dimensions:
                    del dimensions['db']

                dimensions.update({d[0][1]: d[1] for d in zip(desc, row[:len(desc)])})

                # [(metric-map, value), (metric-map, value), ...]
                # metric-map is: (dd_name, "rate"|"gauge")
                # shift the results since the first columns will be the "descriptors"
                values = zip([scope['metrics'][c] for c in cols], row[len(desc):])

                # To submit simply call the function for each value v
                # v[0] == (metric_name, submit_function)
                # v[1] == the actual value
                # dimensions are
                [v[0][1](self, v[0][0], v[1], dimensions=dimensions) for v in values]

    def get_connection(self, key, host, port, user, password, dbname, use_cached=True):
        """Get and memorize connections to instances.

        """
        if key in self.dbs and use_cached:
            return self.dbs[key]

        elif host != "" and user != "":
            try:
                import psycopg2 as pg
            except ImportError:
                raise ImportError(
                    "psycopg2 library cannot be imported. Please check the installation instruction on the Datadog Website.")

            if host == 'localhost' and password == '':
                # Use ident method
                connection = pg.connect("user=%s dbname=%s" % (user, dbname))
            elif port != '':
                connection = pg.connect(host=host, port=port, user=user,
                                        password=password, database=dbname)
            else:
                connection = pg.connect(host=host, user=user, password=password,
                                        database=dbname)
        else:
            if not host:
                raise CheckException("Please specify a Postgres host to connect to.")
            elif not user:
                raise CheckException("Please specify a user to connect to Postgres as.")

        try:
            connection.autocommit = True
        except AttributeError:
            # connection.autocommit was added in version 2.4.2
            from psycopg2.extensions import ISOLATION_LEVEL_AUTOCOMMIT
            connection.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)

        self.dbs[key] = connection
        return connection

    def check(self, instance):
        host = instance.get('host', '')
        port = instance.get('port', '')
        user = instance.get('username', '')
        password = instance.get('password', '')
        dbname = instance.get('dbname', 'postgres')
        relations = instance.get('relations', [])

        key = '%s:%s:%s' % (host, port, dbname)
        db = self.get_connection(key, host, port, user, password, dbname)

        # preset dimensions to the database name
        dimensions = self._set_dimensions({'db': dbname, 'service': 'postgres'}, instance)

        # Check version
        version = self._get_version(key, db)
        self.log.debug("Running check against version %s" % version)

        # Collect metrics
        try:
            self._collect_stats(key, db, dimensions, relations)
        except ShouldRestartException:
            self.log.info("Resetting the connection")
            db = self.get_connection(key, host, port, user, password, dbname, use_cached=False)
            self._collect_stats(key, db, dimensions, relations)
